<?php
/**
 * Certificate Diagnostic Test
 * 
 * This script tests the certificate files and OpenSSL functionality
 * to diagnose issues with PKPass generation.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Start time
$startTime = microtime(true);

// HTML header
echo "<!DOCTYPE html>
<html>
<head>
    <meta charset='UTF-8'>
    <meta name='viewport' content='width=device-width, initial-scale=1.0'>
    <title>Certificate Diagnostic Test</title>
    <style>
        body { font-family: -apple-system, system-ui, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif; line-height: 1.6; color: #333; max-width: 1000px; margin: 0 auto; padding: 20px; }
        h1 { color: #0071e3; }
        h2 { margin-top: 30px; border-bottom: 1px solid #eee; padding-bottom: 10px; }
        pre { background: #f5f5f5; padding: 15px; border-radius: 5px; overflow-x: auto; white-space: pre-wrap; }
        .card { background: #fff; border-radius: 8px; box-shadow: 0 2px 10px rgba(0,0,0,0.1); padding: 20px; margin: 20px 0; }
        .success { color: #388e3c; }
        .error { color: #c62828; }
        .warning { color: #ff8f00; }
        .info { color: #0277bd; }
        .details { margin-left: 20px; border-left: 3px solid #ddd; padding-left: 20px; }
        table { border-collapse: collapse; width: 100%; margin: 15px 0; }
        th, td { text-align: left; padding: 8px; }
        th { background-color: #f5f5f5; }
        tr:nth-child(even) { background-color: #f9f9f9; }
        .button { display: inline-block; padding: 10px 15px; background: #0071e3; color: white; text-decoration: none; border-radius: 5px; margin-top: 10px; }
        .button:hover { background: #005bb5; }
        .progress-container { width: 100%; background-color: #f1f1f1; border-radius: 5px; margin: 10px 0; }
        .progress-bar { height: 20px; border-radius: 5px; background-color: #4CAF50; text-align: center; line-height: 20px; color: white; }
    </style>
</head>
<body>
    <h1>Certificate Diagnostic Test</h1>
    <p>This tool performs a comprehensive test of your certificate files and OpenSSL functionality to help diagnose issues with PKPass generation.</p>";

// Log to browser and file
function log_message($message, $type = 'info') {
    static $logFile = null;
    if ($logFile === null) {
        $logDir = __DIR__ . '/../logs';
        if (!file_exists($logDir)) {
            mkdir($logDir, 0755, true);
        }
        $logFile = $logDir . '/certificate_diagnostic.log';
        file_put_contents($logFile, "Certificate Diagnostic Test: " . date('Y-m-d H:i:s') . "\n\n", FILE_APPEND);
    }
    
    // Log to file
    file_put_contents($logFile, "[" . date('H:i:s') . "] " . strip_tags($message) . "\n", FILE_APPEND);
    
    // Display in browser
    $class = '';
    switch ($type) {
        case 'success': $class = 'class="success"'; break;
        case 'error': $class = 'class="error"'; break;
        case 'warning': $class = 'class="warning"'; break;
        case 'info': $class = 'class="info"'; break;
    }
    
    echo "<p $class>" . $message . "</p>";
}

// 1. PHP and OpenSSL Version Check
echo "<div class='card'>";
echo "<h2>PHP and OpenSSL Environment</h2>";

$phpVersion = phpversion();
log_message("PHP Version: " . $phpVersion);

if (function_exists('openssl_get_cert_locations')) {
    $opensslLocations = openssl_get_cert_locations();
    log_message("OpenSSL Default Locations:", 'info');
    echo "<div class='details'>";
    foreach ($opensslLocations as $key => $location) {
        echo "<p><strong>$key:</strong> $location</p>";
    }
    echo "</div>";
}

$opensslVersion = OPENSSL_VERSION_TEXT;
log_message("OpenSSL Version: " . $opensslVersion);

$opensslCiphers = openssl_get_cipher_methods();
log_message("Available Cipher Methods: " . count($opensslCiphers) . " methods");

$extensions = get_loaded_extensions();
$requiredExtensions = ['openssl', 'zip', 'json', 'mbstring'];
echo "<table>";
echo "<tr><th>Extension</th><th>Status</th><th>Version</th></tr>";
foreach ($requiredExtensions as $ext) {
    $loaded = in_array($ext, $extensions);
    $version = $loaded && function_exists('phpversion') ? phpversion($ext) : 'N/A';
    $status = $loaded ? '<span class="success">Loaded</span>' : '<span class="error">Missing</span>';
    echo "<tr><td>$ext</td><td>$status</td><td>$version</td></tr>";
    
    if (!$loaded && $ext === 'openssl') {
        log_message("Error: OpenSSL extension is not loaded. This is required for PKPass generation.", 'error');
    }
}
echo "</table>";

echo "</div>";

// 2. Certificate File Analysis
echo "<div class='card'>";
echo "<h2>Certificate File Analysis</h2>";

$certificatePath = __DIR__ . '/../certs/Certificates.p12';
$wwdrCertPath = __DIR__ . '/../certs/AppleWWDRMPCA1G1.cer';

// Check certificate existence
if (file_exists($certificatePath)) {
    $filesize = filesize($certificatePath);
    $filetime = date('Y-m-d H:i:s', filemtime($certificatePath));
    log_message("Certificate file found: " . $filesize . " bytes, last modified: " . $filetime, 'success');
    
    // Try to analyze the file
    $fileContent = file_get_contents($certificatePath);
    if ($fileContent) {
        log_message("Successfully read certificate file", 'success');
        
        // Check file headers/magic bytes to confirm it's a PKCS12 file
        $fileSignature = bin2hex(substr($fileContent, 0, 4));
        if ($fileSignature === '30820') {
            log_message("File signature appears to be correct for a PKCS12 file", 'success');
        } else {
            log_message("Warning: File signature doesn't match expected PKCS12 format (got: " . $fileSignature . ")", 'warning');
        }
        
        // Try with different passwords
        $possiblePasswords = [
            'T@pple9696',
            'Tapple9696',
            '',  // Empty password
            'apple',
            '1234',
        ];
        
        $passwordSuccess = false;
        foreach ($possiblePasswords as $password) {
            echo "<p>Testing password: <code>" . ($password === '' ? '(empty)' : htmlspecialchars($password)) . "</code> - ";
            $result = openssl_pkcs12_read($fileContent, $certs, $password);
            if ($result) {
                echo "<span class='success'>Success!</span></p>";
                log_message("Certificate successfully read with password: " . ($password === '' ? '(empty)' : $password), 'success');
                $passwordSuccess = true;
                
                // Show certificate details
                echo "<div class='details'>";
                echo "<p><strong>Certificate Components:</strong></p>";
                echo "<ul>";
                foreach (array_keys($certs) as $key) {
                    echo "<li>" . htmlspecialchars($key) . "</li>";
                }
                echo "</ul>";
                
                // Extract and display certificate info
                if (isset($certs['cert'])) {
                    $certInfo = openssl_x509_parse($certs['cert']);
                    if ($certInfo) {
                        echo "<p><strong>Certificate Subject:</strong> " . htmlspecialchars($certInfo['subject']['CN']) . "</p>";
                        echo "<p><strong>Issued By:</strong> " . htmlspecialchars($certInfo['issuer']['CN']) . "</p>";
                        echo "<p><strong>Valid From:</strong> " . date('Y-m-d H:i:s', $certInfo['validFrom_time_t']) . "</p>";
                        echo "<p><strong>Valid Until:</strong> " . date('Y-m-d H:i:s', $certInfo['validTo_time_t']) . "</p>";
                        
                        // Check if the certificate is expired
                        if (time() > $certInfo['validTo_time_t']) {
                            log_message("Certificate has expired on " . date('Y-m-d', $certInfo['validTo_time_t']), 'error');
                        } else {
                            $daysLeft = floor(($certInfo['validTo_time_t'] - time()) / 86400);
                            log_message("Certificate is valid for " . $daysLeft . " more days", 'success');
                        }
                    }
                }
                
                echo "</div>";
                break;
            } else {
                $error = openssl_error_string();
                echo "<span class='error'>Failed - " . htmlspecialchars($error) . "</span></p>";
            }
        }
        
        if (!$passwordSuccess) {
            log_message("Failed to read the certificate with any of the test passwords", 'error');
            log_message("You may need to export a new certificate from Apple Developer Portal with a known password", 'warning');
        }
    } else {
        log_message("Failed to read certificate file contents", 'error');
    }
} else {
    log_message("Certificate file not found at: " . $certificatePath, 'error');
}

// Check WWDR certificate
if (file_exists($wwdrCertPath)) {
    $filesize = filesize($wwdrCertPath);
    $filetime = date('Y-m-d H:i:s', filemtime($wwdrCertPath));
    log_message("WWDR certificate file found: " . $filesize . " bytes, last modified: " . $filetime, 'success');
    
    // Try to read and analyze the WWDR certificate
    $wwdrData = file_get_contents($wwdrCertPath);
    if ($wwdrData) {
        log_message("Successfully read WWDR certificate file", 'success');
        
        // Convert DER to PEM
        $pem = "-----BEGIN CERTIFICATE-----\n";
        $pem .= chunk_split(base64_encode($wwdrData), 64, "\n");
        $pem .= "-----END CERTIFICATE-----\n";
        
        // Try to parse the certificate
        $cert = openssl_x509_read($pem);
        if ($cert) {
            log_message("Successfully parsed WWDR certificate", 'success');
            
            // Display certificate details
            $certInfo = openssl_x509_parse($cert);
            if ($certInfo) {
                echo "<div class='details'>";
                echo "<p><strong>WWDR Certificate Subject:</strong> " . htmlspecialchars($certInfo['subject']['CN']) . "</p>";
                echo "<p><strong>WWDR Issued By:</strong> " . htmlspecialchars($certInfo['issuer']['CN']) . "</p>";
                echo "<p><strong>Valid From:</strong> " . date('Y-m-d H:i:s', $certInfo['validFrom_time_t']) . "</p>";
                echo "<p><strong>Valid Until:</strong> " . date('Y-m-d H:i:s', $certInfo['validTo_time_t']) . "</p>";
                echo "</div>";
                
                // Check if the certificate is expired
                if (time() > $certInfo['validTo_time_t']) {
                    log_message("WWDR Certificate has expired on " . date('Y-m-d', $certInfo['validTo_time_t']), 'error');
                    log_message("You need to download a new WWDR certificate from Apple: <a href='https://www.apple.com/certificateauthority/' target='_blank'>Apple Certificate Authority</a>", 'warning');
                } else {
                    $daysLeft = floor(($certInfo['validTo_time_t'] - time()) / 86400);
                    log_message("WWDR Certificate is valid for " . $daysLeft . " more days", 'success');
                }
            }
        } else {
            $error = openssl_error_string();
            log_message("Failed to parse WWDR certificate: " . $error, 'error');
        }
    } else {
        log_message("Failed to read WWDR certificate file contents", 'error');
    }
} else {
    log_message("WWDR certificate file not found at: " . $wwdrCertPath, 'error');
}

echo "</div>";

// 3. Signature Creation Test
echo "<div class='card'>";
echo "<h2>Signature Creation Test</h2>";

// Create a test directory
$testDir = __DIR__ . '/../temp/cert_test_' . uniqid();
if (!file_exists($testDir)) {
    mkdir($testDir, 0755, true);
}

// Create a test file to sign
$testFile = $testDir . '/manifest.json';
$testData = json_encode(['test.txt' => sha1('test content')]);
file_put_contents($testFile, $testData);
log_message("Created test manifest file: " . $testFile);

// If password was discovered above, use it
$certPassword = isset($passwordSuccess) && $passwordSuccess ? $password : 'T@pple9696';

// Try to sign the test file
if (file_exists($certificatePath) && file_exists($wwdrCertPath)) {
    
    // Extract cert and key from P12
    $pkcs12 = file_get_contents($certificatePath);
    if (openssl_pkcs12_read($pkcs12, $certs, $certPassword)) {
        log_message("Successfully extracted certificate and key from P12 file", 'success');
        
        // Save certificate and key to temporary files
        $certFile = $testDir . '/cert.pem';
        $keyFile = $testDir . '/key.pem';
        file_put_contents($certFile, $certs['cert']);
        file_put_contents($keyFile, $certs['pkey']);
        
        // Convert WWDR to PEM
        $wwdrData = file_get_contents($wwdrCertPath);
        $wwdrPem = "-----BEGIN CERTIFICATE-----\n";
        $wwdrPem .= chunk_split(base64_encode($wwdrData), 64, "\n");
        $wwdrPem .= "-----END CERTIFICATE-----\n";
        $wwdrFile = $testDir . '/wwdr.pem';
        file_put_contents($wwdrFile, $wwdrPem);
        
        // Try to sign the test file
        $signaturePath = $testDir . '/signature.p7s';
        $result = openssl_pkcs7_sign(
            $testFile,
            $signaturePath,
            "file://$certFile",
            ["file://$keyFile", $certPassword],
            [],
            PKCS7_BINARY | PKCS7_DETACHED,
            "file://$wwdrFile"
        );
        
        if ($result && file_exists($signaturePath) && filesize($signaturePath) > 0) {
            log_message("Successfully created a PKCS7 signature", 'success');
            
            // Extract DER data
            $signatureContent = file_get_contents($signaturePath);
            if (preg_match('~Content-Disposition:[^\n]+\s*?([A-Za-z0-9+=/\r\n]+)\s*?-----~s', $signatureContent, $matches)) {
                $signature = preg_replace('/\s+/', '', $matches[1]);
                $derSignature = base64_decode($signature);
                
                if ($derSignature) {
                    $derPath = $testDir . '/signature.der';
                    file_put_contents($derPath, $derSignature);
                    $derSize = strlen($derSignature);
                    log_message("Successfully extracted DER signature: " . $derSize . " bytes", 'success');
                    
                    // Verify the signature format
                    $derHeader = bin2hex(substr($derSignature, 0, 4));
                    log_message("DER signature header: " . $derHeader);
                    
                    if (strpos($derHeader, '3082') === 0) {
                        log_message("DER signature format appears to be correct", 'success');
                    } else {
                        log_message("Warning: DER signature format does not match expected pattern", 'warning');
                    }
                } else {
                    log_message("Failed to decode base64 signature data", 'error');
                }
            } else {
                log_message("Failed to extract signature data from PEM format", 'error');
                
                // Try alternative extraction
                $fullSignature = file_get_contents($signaturePath);
                $pos = strpos($fullSignature, "Content-Type: application/x-pkcs7-signature");
                if ($pos !== false) {
                    // Find the end of the headers (double newline)
                    $pos = strpos($fullSignature, "\r\n\r\n", $pos);
                    if ($pos !== false) {
                        $pos += 4; // Move past the double newline
                        $binaryData = substr($fullSignature, $pos);
                        
                        if (!empty($binaryData)) {
                            $endPos = strpos($binaryData, "\r\n--");
                            if ($endPos !== false) {
                                $binaryData = substr($binaryData, 0, $endPos);
                            }
                            
                            $decoded = base64_decode(trim($binaryData));
                            if ($decoded) {
                                $altDerPath = $testDir . '/signature_alt.der';
                                file_put_contents($altDerPath, $decoded);
                                $altDerSize = strlen($decoded);
                                log_message("Successfully extracted DER signature using alternative method: " . $altDerSize . " bytes", 'success');
                            } else {
                                log_message("Failed to decode alternative signature data", 'error');
                            }
                        }
                    }
                }
            }
        } else {
            $error = openssl_error_string();
            log_message("Failed to create signature: " . $error, 'error');
            
            // Try direct approach
            $result = openssl_pkcs7_sign(
                $testFile,
                $signaturePath,
                $certs['cert'],
                [$certs['pkey'], $certPassword],
                [],
                PKCS7_BINARY | PKCS7_DETACHED,
                $wwdrPem
            );
            
            if ($result && file_exists($signaturePath) && filesize($signaturePath) > 0) {
                log_message("Successfully created a PKCS7 signature using direct certificate data", 'success');
            } else {
                $error = openssl_error_string();
                log_message("Failed to create signature using direct approach: " . $error, 'error');
            }
        }
        
    } else {
        $error = openssl_error_string();
        log_message("Could not read certificate with provided password: " . $error, 'error');
    }
} else {
    log_message("Cannot perform signature test - missing certificate files", 'error');
}

// Cleanup test files
array_map('unlink', glob("$testDir/*"));
rmdir($testDir);
log_message("Cleaned up test files");

echo "</div>";

// 4. Recommendations
echo "<div class='card'>";
echo "<h2>Recommendations and Next Steps</h2>";

echo "<ul>";
if (!file_exists($certificatePath)) {
    echo "<li>You need to obtain a pass type ID certificate from the Apple Developer Portal</li>";
    echo "<li>Export it as a .p12 file with a simple password</li>";
}

if (!file_exists($wwdrCertPath)) {
    echo "<li>Download the Apple Worldwide Developer Relations certificate from <a href='https://www.apple.com/certificateauthority/' target='_blank'>Apple Certificate Authority</a></li>";
}

if (isset($passwordSuccess) && !$passwordSuccess) {
    echo "<li>Export your certificate again with a known password</li>";
    echo "<li>When exporting from Keychain Access, choose 'Personal Information Exchange (.p12)' format</li>";
    echo "<li>Use a simple password without special characters</li>";
}

if (isset($result) && !$result) {
    echo "<li>Your OpenSSL configuration might not support the algorithms used in your certificate</li>";
    echo "<li>Consider using a newer version of PHP with updated OpenSSL</li>";
    echo "<li>Try exporting your certificate with different encryption settings</li>";
}

echo "<li>Make sure your pass type identifier in the pass.json file matches your certificate</li>";
echo "<li>Ensure your team identifier matches your Apple Developer account</li>";
echo "</ul>";

// Display links to other test tools
echo "<p><strong>Additional Tests:</strong></p>";
echo "<a href='test_certificates_live.php' class='button'>Run Live Certificate Test</a> ";
echo "<a href='simple_pkpass_test.php' class='button'>Run Simple PKPass Test</a> ";
echo "<a href='generate_proper_pkpass.php?card_number=LC202503253423' class='button'>Generate Real Loyalty Card</a>";

echo "</div>";

// 5. Technical Information (for developers)
echo "<div class='card'>";
echo "<h2>Technical Information</h2>";

echo "<details>";
echo "<summary>PHP Information (OpenSSL Section)</summary>";
echo "<pre>";
ob_start();
phpinfo(INFO_MODULES);
$phpinfo = ob_get_clean();

// Extract OpenSSL section
if (preg_match('/<h2><a name="module_openssl">.*?<\/table>/s', $phpinfo, $matches)) {
    echo htmlspecialchars($matches[0]);
} else {
    echo "OpenSSL section not found in phpinfo()";
}
echo "</pre>";
echo "</details>";

// OpenSSL test
echo "<details>";
echo "<summary>OpenSSL Test Output</summary>";
echo "<pre>";
$opensslOutput = shell_exec('openssl version -a 2>&1');
echo htmlspecialchars($opensslOutput ?: "OpenSSL command not available");
echo "</pre>";
echo "</details>";

// Current certificate paths
echo "<p><strong>Certificate Paths:</strong></p>";
echo "<ul>";
echo "<li>P12 Certificate: " . htmlspecialchars($certificatePath) . "</li>";
echo "<li>WWDR Certificate: " . htmlspecialchars($wwdrCertPath) . "</li>";
echo "</ul>";

// Execution time
$endTime = microtime(true);
$executionTime = round($endTime - $startTime, 2);
echo "<p><strong>Execution Time:</strong> " . $executionTime . " seconds</p>";

echo "</div>";

echo "</body>
</html>";
?> 