<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include database connection
require_once('../includes/functions.php');

// Handle device registration for push notifications if POST request
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $inputData = file_get_contents('php://input');
    $data = json_decode($inputData, true);
    
    // Check if it's a device registration request
    if (isset($data['pushToken']) && isset($data['serialNumber'])) {
        handleDeviceRegistration($data);
        exit;
    }
    
    // Check if it's a device unregistration request
    if (isset($data['deviceLibraryIdentifier']) && isset($data['serialNumber']) && isset($data['passTypeIdentifier'])) {
        handleDeviceUnregistration($data);
        exit;
    }
    
    // If it's not a valid push notification request
    header('HTTP/1.0 400 Bad Request');
    echo 'Invalid request';
    exit;
}

// Check if file parameter is provided for normal download flow
if (!isset($_GET['file']) || empty($_GET['file'])) {
    die('File parameter is required');
}

// Sanitize filename to prevent directory traversal attacks
$filename = basename($_GET['file']);
$filepath = __DIR__ . '/../passes/' . $filename;

// Check if file exists
if (!file_exists($filepath)) {
    die('File not found: ' . htmlspecialchars($filename));
}

// Check if file is readable
if (!is_readable($filepath)) {
    die('File is not readable: ' . htmlspecialchars($filename));
}

// Get file size
$filesize = filesize($filepath);
if ($filesize === false) {
    die('Failed to get file size');
}

// Set appropriate headers for .pkpass file
header('Content-Type: application/vnd.apple.pkpass');
header('Content-Disposition: attachment; filename="' . $filename . '"');
header('Content-Length: ' . $filesize);
header('Pragma: no-cache');
header('Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0');
header('Expires: 0');
header('X-Apple-Wallet-Compatible: 1');

// Output file content
$handle = fopen($filepath, 'rb');
if ($handle === false) {
    die('Failed to open file for reading');
}

// Stream file in chunks to avoid memory issues with large files
while (!feof($handle)) {
    echo fread($handle, 8192);
    flush();
}
fclose($handle);
exit;

/**
 * Handles device registration for pass updates
 * @param array $data Registration data
 */
function handleDeviceRegistration($data) {
    $pushToken = $data['pushToken'];
    $serialNumber = $data['serialNumber']; // Card number
    $deviceLibraryIdentifier = $data['deviceLibraryIdentifier'];
    $passTypeIdentifier = isset($data['passTypeIdentifier']) ? $data['passTypeIdentifier'] : 'pass.pointify.pk';
    
    // Store device registration in database
    $conn = getDbConnection();
    
    // Check if registration already exists
    $stmt = $conn->prepare("SELECT id FROM pass_registrations 
                            WHERE device_library_identifier = :device_id 
                            AND pass_type_identifier = :pass_type 
                            AND serial_number = :serial_number");
    $stmt->bindParam(':device_id', $deviceLibraryIdentifier);
    $stmt->bindParam(':pass_type', $passTypeIdentifier);
    $stmt->bindParam(':serial_number', $serialNumber);
    $stmt->execute();
    
    if ($stmt->fetch()) {
        // Update existing registration
        $stmt = $conn->prepare("UPDATE pass_registrations SET 
                                push_token = :push_token,
                                updated_at = NOW()
                                WHERE device_library_identifier = :device_id 
                                AND pass_type_identifier = :pass_type 
                                AND serial_number = :serial_number");
    } else {
        // Create new registration
        $stmt = $conn->prepare("INSERT INTO pass_registrations 
                                (device_library_identifier, pass_type_identifier, serial_number, push_token, created_at, updated_at) 
                                VALUES 
                                (:device_id, :pass_type, :serial_number, :push_token, NOW(), NOW())");
    }
    
    $stmt->bindParam(':device_id', $deviceLibraryIdentifier);
    $stmt->bindParam(':pass_type', $passTypeIdentifier);
    $stmt->bindParam(':serial_number', $serialNumber);
    $stmt->bindParam(':push_token', $pushToken);
    
    if ($stmt->execute()) {
        header('HTTP/1.0 200 OK');
        echo json_encode(['status' => 'success']);
    } else {
        header('HTTP/1.0 500 Internal Server Error');
        echo json_encode(['status' => 'error', 'message' => 'Failed to register device']);
    }
}

/**
 * Handles device unregistration
 * @param array $data Unregistration data
 */
function handleDeviceUnregistration($data) {
    $deviceLibraryIdentifier = $data['deviceLibraryIdentifier'];
    $serialNumber = $data['serialNumber'];
    $passTypeIdentifier = $data['passTypeIdentifier'];
    
    // Remove device registration from database
    $conn = getDbConnection();
    $stmt = $conn->prepare("DELETE FROM pass_registrations 
                            WHERE device_library_identifier = :device_id 
                            AND pass_type_identifier = :pass_type 
                            AND serial_number = :serial_number");
    
    $stmt->bindParam(':device_id', $deviceLibraryIdentifier);
    $stmt->bindParam(':pass_type', $passTypeIdentifier);
    $stmt->bindParam(':serial_number', $serialNumber);
    
    if ($stmt->execute()) {
        header('HTTP/1.0 200 OK');
        echo json_encode(['status' => 'success']);
    } else {
        header('HTTP/1.0 500 Internal Server Error');
        echo json_encode(['status' => 'error', 'message' => 'Failed to unregister device']);
    }
} 