<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include database connection
require_once('../includes/functions.php');

// Check if card number is provided
if (!isset($_GET['card_number']) || empty($_GET['card_number'])) {
    header('HTTP/1.0 400 Bad Request');
    echo 'Card number is required';
    exit;
}

$cardNumber = $_GET['card_number'];

// Get card details using the same query as in card.php
$conn = getDbConnection();
$stmt = $conn->prepare("
    SELECT lc.*, c.name, c.email, l.name as location_name, l.logo_url, l.reward_threshold, l.reward_description
    FROM loyalty_cards lc
    JOIN customers c ON lc.customer_id = c.id
    JOIN locations l ON c.location_id = l.id
    WHERE lc.card_number = :card_number
");
$stmt->bindParam(':card_number', $cardNumber);
$stmt->execute();
$card = $stmt->fetch(PDO::FETCH_ASSOC);

if (!$card) {
    header('HTTP/1.0 404 Not Found');
    echo 'Card not found';
    exit;
}

// Set paths
$certDir = __DIR__ . '/../certs/';
$signerCertPath = $certDir . 'signerCert.pem';
$signerKeyPath = $certDir . 'signerKey.pem';
$wwdrCertPath = $certDir . 'Apple Worldwide Developer Relations Certification Authority.pem';
$keyPassphrase = 'test'; // The passphrase from your test script
$passesDir = __DIR__ . '/../passes/';
$tempDir = $passesDir . 'tmp/';
$passFile = $passesDir . $cardNumber . '.pkpass';

// Create temp directory for this pass
$passDir = $tempDir . uniqid('pass_');
if (!is_dir($tempDir)) mkdir($tempDir, 0777, true);
if (!is_dir($passDir)) mkdir($passDir, 0777, true);

// Create the pass.json directly (don't use template)
$pass = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.pointify.pk',
    'teamIdentifier' => 'C7CH6P3GLR',
    'organizationName' => 'Pointify',
    'description' => 'Loyalty Card',
    'serialNumber' => $cardNumber,
    'foregroundColor' => 'rgb(67, 97, 238)',
    'backgroundColor' => 'rgb(6, 6, 6)',
    'labelColor' => 'rgb(67, 97, 238)',
    'logoText' => 'Pointify Loyalty',
    'webServiceURL' => 'https://dev.trimscloud.com/customer/webservice.php',
    'authenticationToken' => md5($cardNumber . 'pointify_secret_salt'),
    'storeCard' => [
        'headerFields' => [[
            'key' => 'balance',
            'label' => 'STAMPS',
            'value' => (string)$card['stamp_count']
        ]],
        'primaryFields' => [[
            'key' => 'member',
            'label' => 'MEMBER',
            'value' => $card['name']
        ]],
        'secondaryFields' => [[
            'key' => 'subtitle',
            'label' => 'CARD NUMBER',
            'value' => $cardNumber
        ]],
        'auxiliaryFields' => [[
            'key' => 'reward',
            'label' => 'REWARD',
            'value' => 'Buy ' . ($card['reward_threshold'] - 1) . ' Get 1 Free'
        ]],
        'backFields' => [
            [
                'key' => 'terms',
                'label' => 'TERMS AND CONDITIONS',
                'value' => 'Collect stamps with every purchase. Complete your card to earn rewards.'
            ],
            [
                'key' => 'website',
                'label' => 'WEBSITE',
                'value' => 'https://pointify.com'
            ]
        ]
    ],
    'barcode' => [
        'message' => $cardNumber,
        'format' => 'PKBarcodeFormatPDF417',
        'messageEncoding' => 'iso-8859-1',
        'altText' => $cardNumber
    ]
];

// Write pass.json
file_put_contents($passDir . '/pass.json', json_encode($pass));

// Copy required images from the assets/pass_images directory
$passImagesDir = __DIR__ . '/../assets/pass_images/';
$requiredImages = ['icon.png', 'icon@2x.png', 'logo.png', 'logo@2x.png'];
$optionalImages = ['strip.png', 'strip@2x.png', 'thumbnail.png', 'thumbnail@2x.png'];

foreach ($requiredImages as $img) {
    if (file_exists($passImagesDir . $img)) {
        copy($passImagesDir . $img, $passDir . '/' . $img);
    } else {
        die("Missing required image: $img");
    }
}

// Copy optional images if they exist
foreach ($optionalImages as $img) {
    if (file_exists($passImagesDir . $img)) {
        copy($passImagesDir . $img, $passDir . '/' . $img);
    }
}

// Create manifest.json (contains SHA1 hash of each file in the bundle)
$manifest = [];
$files = glob($passDir . '/*');
foreach ($files as $file) {
    if (is_file($file) && basename($file) != 'manifest.json' && basename($file) != 'signature') {
        $manifest[basename($file)] = sha1_file($file);
    }
}
file_put_contents($passDir . '/manifest.json', json_encode($manifest));

// Sign the manifest
$manifestData = file_get_contents($passDir . '/manifest.json');
$tempManifestFile = $tempDir . 'manifest_' . uniqid() . '.json';
file_put_contents($tempManifestFile, $manifestData);

// Load key and certificate
$pkeyid = openssl_pkey_get_private(file_get_contents($signerKeyPath), $keyPassphrase);
$cert = file_get_contents($signerCertPath);

if (!$pkeyid) {
    die("Error loading private key: " . openssl_error_string());
}

// Sign
if (!openssl_pkcs7_sign(
    $tempManifestFile, 
    $passDir . '/signature',
    $cert,
    [$pkeyid, $keyPassphrase],
    [],
    PKCS7_BINARY | PKCS7_DETACHED,
    $wwdrCertPath
)) {
    die("Error signing manifest: " . openssl_error_string());
}

// Extract signature
$signatureContent = file_get_contents($passDir . '/signature');
$signature = '';
if (preg_match('~Content-Disposition:[^\n]+\s*?([A-Za-z0-9+=/\r\n]+)\s*?--~', $signatureContent, $matches)) {
    $signature = base64_decode(preg_replace('~\s+~', '', $matches[1]));
} elseif (preg_match('~\n\n(.+)~s', $signatureContent, $matches)) {
    $signature = base64_decode(trim($matches[1]));
} else {
    die("Could not extract signature data");
}

file_put_contents($passDir . '/signature', $signature);

// Create .pkpass (zip archive)
$zip = new ZipArchive();
if ($zip->open($passFile, ZipArchive::CREATE | ZipArchive::OVERWRITE) !== TRUE) {
    die("Cannot create ZIP archive");
}

// Add all necessary files
$filesToZip = ['pass.json', 'manifest.json', 'signature'];
foreach ($requiredImages as $img) {
    $filesToZip[] = $img;
}
foreach ($optionalImages as $img) {
    if (file_exists($passDir . '/' . $img)) {
        $filesToZip[] = $img;
    }
}

foreach ($filesToZip as $file) {
    if (file_exists($passDir . '/' . $file)) {
        $zip->addFile($passDir . '/' . $file, $file);
    }
}

$zip->close();

// Clean up temporary files
unlink($tempManifestFile);
foreach (glob($passDir . '/*') as $file) {
    unlink($file);
}
rmdir($passDir);

// Redirect to the download script
header("Location: download_pass.php?file=" . urlencode(basename($passFile)));
exit; 