<?php
// Enable maximum error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Define absolute paths
define('BASE_DIR', '/home/trimscloud/public_html/dev.trimscloud.com/');
define('CERT_DIR', BASE_DIR . 'certs/');
define('IMAGE_DIR', BASE_DIR . 'assets/pass_images/');
define('LOG_FILE', BASE_DIR . 'logs/pkpass_errors.log');

// Clear output buffers
while (ob_get_level()) ob_end_clean();

function log_error($message) {
    file_put_contents(LOG_FILE, date('Y-m-d H:i:s') . ' - ' . $message . PHP_EOL, FILE_APPEND);
}

try {
    // Validate input
    if (!isset($_GET['card_number'])) {
        throw new Exception('Missing card_number parameter');
    }

    $cardNumber = $_GET['card_number'];
    log_error("Processing card: $cardNumber");

    // Load dependencies
    require_once BASE_DIR . 'includes/functions.php';
    require_once BASE_DIR . 'vendor/autoload.php';

    // Initialize PKPass with DIRECT P12 USAGE (confirmed working)
    $pass = new PKPass\PKPass(
        CERT_DIR . 'Certificates.p12',  // Using the P12 file directly
        'T@pple9696',                   // CONFIRMED WORKING PASSWORD
        CERT_DIR . 'Apple Worldwide Developer Relations Certification Authority.pem'
    );

    // Get card data
    $conn = getDbConnection();
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, l.name as location_name, l.reward_threshold, lc.stamp_count
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = ?
    ");
    $stmt->execute([$cardNumber]);
    $card = $stmt->fetch(PDO::FETCH_ASSOC);

    if (!$card) {
        throw new Exception('Invalid card number: ' . $cardNumber);
    }

    // Configure pass data
    $passData = [
        'formatVersion' => 1,
        'passTypeIdentifier' => 'pass.com.trimscloud.loyalty',
        'serialNumber' => $cardNumber,
        'teamIdentifier' => 'C7CH6P3GLR',
        'organizationName' => $card['location_name'],
        'description' => $card['location_name'] . ' Loyalty Card',
        'logoText' => $card['location_name'],
        'foregroundColor' => 'rgb(67, 97, 238)',
        'backgroundColor' => 'rgb(255, 255, 255)',
        'labelColor' => 'rgb(67, 97, 238)',
        'storeCard' => [
            'primaryFields' => [[
                'key' => 'points',
                'label' => 'STAMPS',
                'value' => $card['stamp_count'] . ' of ' . $card['reward_threshold']
            ]],
            'secondaryFields' => [[
                'key' => 'name',
                'label' => 'MEMBER',
                'value' => $card['name']
            ]]
        ],
        'barcode' => [
            'message' => $cardNumber,
            'format' => 'PKBarcodeFormatPDF417',
            'messageEncoding' => 'iso-8859-1'
        ]
    ];

    $pass->setData($passData);

    // Add required images
    $images = [
        'icon.png' => 'icon.png',
        'logo.png' => 'logo.png'
    ];

    foreach ($images as $file => $dest) {
        $path = IMAGE_DIR . $file;
        if (!file_exists($path)) {
            throw new Exception("Missing required image: $file");
        }
        $pass->addFile($path, $dest);
    }

    // Generate the pass
    $pkpass = $pass->create();
    if (!$pkpass) {
        throw new Exception('Failed to generate pass file');
    }

    // Output headers
    header('Pragma: no-cache');
    header('Cache-Control: no-cache');
    header('Content-Type: application/vnd.apple.pkpass');
    header('Content-Disposition: attachment; filename="loyalty_'.$cardNumber.'.pkpass"');
    header('Content-Length: ' . strlen($pkpass));
    
    echo $pkpass;
    log_error("Successfully generated pass for card: $cardNumber");
    exit;

} catch (Exception $e) {
    // Clean output buffer
    while (ob_get_level()) ob_end_clean();
    
    // Log and display error
    $errorMsg = "ERROR: " . $e->getMessage();
    log_error($errorMsg);
    header('Content-Type: text/plain');
    http_response_code(500);
    echo $errorMsg;
    exit;
}