<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Start logging to a file to track execution
$logFile = __DIR__ . '/../logs/pkpass_debug_pem.log';
file_put_contents($logFile, "Starting PKPass generation with PEM certificates: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

require_once __DIR__ . '/../includes/functions.php';
file_put_contents($logFile, "Loaded functions.php\n", FILE_APPEND);

require_once __DIR__ . '/../vendor/autoload.php';
file_put_contents($logFile, "Loaded autoload.php\n", FILE_APPEND);

// Check if PEM certificates exist
$pemCertPath = __DIR__ . '/../certs/certificate.pem';
$pemKeyPath = __DIR__ . '/../certs/key.pem';
$pemWWDRPath = __DIR__ . '/../certs/wwdr.pem';

if (!file_exists($pemCertPath) || !file_exists($pemKeyPath) || !file_exists($pemWWDRPath)) {
    file_put_contents($logFile, "PEM certificates not found. Please run convert_certificates.php first.\n", FILE_APPEND);
    die("PEM certificates not found. Please run <a href='convert_certificates.php'>convert_certificates.php</a> first.");
}

file_put_contents($logFile, "PEM certificates found\n", FILE_APPEND);

// Define a custom PKPass class that uses PEM certificates
class CustomPKPass {
    private $certPath;
    private $keyPath;
    private $wwdrPath;
    private $tempPath;
    private $data;
    private $files = [];
    private $error;
    private $passFilePath;
    
    public function __construct($certPath, $keyPath, $wwdrPath) {
        $this->certPath = $certPath;
        $this->keyPath = $keyPath;
        $this->wwdrPath = $wwdrPath;
    }
    
    public function setTempPath($path) {
        $this->tempPath = $path;
    }
    
    public function setData($data) {
        $this->data = $data;
    }
    
    public function addFile($path, $name) {
        if (!file_exists($path)) {
            throw new Exception("File $path does not exist");
        }
        $this->files[$name] = $path;
    }
    
    public function getError() {
        return $this->error;
    }
    
    public function getPassFilePath() {
        return $this->passFilePath;
    }
    
    public function create($outputToTemp = false) {
        try {
            // Create a temporary directory for working files
            $tempDir = $this->tempPath . '/pkpass_' . uniqid();
            if (!file_exists($tempDir)) {
                mkdir($tempDir, 0755, true);
            }
            
            // Create pass.json with the pass data
            $passJson = json_encode($this->data);
            file_put_contents($tempDir . '/pass.json', $passJson);
            
            // Create a manifest file
            $manifest = [];
            
            // Add pass.json to manifest
            $manifest['pass.json'] = sha1($passJson);
            
            // Add all files to the temp directory and manifest
            foreach ($this->files as $name => $path) {
                $fileContent = file_get_contents($path);
                file_put_contents($tempDir . '/' . $name, $fileContent);
                $manifest[$name] = sha1($fileContent);
            }
            
            // Create manifest.json
            $manifestJson = json_encode($manifest);
            file_put_contents($tempDir . '/manifest.json', $manifestJson);
            
            // Sign the manifest
            $signaturePath = $tempDir . '/signature';
            $manifestPath = $tempDir . '/manifest.json';
            
            $cmd = "openssl smime -sign -signer " . escapeshellarg($this->certPath) . 
                   " -inkey " . escapeshellarg($this->keyPath) . 
                   " -certfile " . escapeshellarg($this->wwdrPath) . 
                   " -in " . escapeshellarg($manifestPath) . 
                   " -out " . escapeshellarg($signaturePath) . 
                   " -outform DER -binary";
            
            exec($cmd, $output, $returnCode);
            
            if ($returnCode !== 0) {
                $this->error = "Failed to sign manifest: " . implode("\n", $output);
                return false;
            }
            
            // Create the pkpass file (which is a zip file)
            $pkpassFilename = $this->tempPath . '/' . uniqid() . '.pkpass';
            
            $zip = new ZipArchive();
            if ($zip->open($pkpassFilename, ZipArchive::CREATE) !== true) {
                $this->error = "Failed to create zip file";
                return false;
            }
            
            // Add all files to the zip
            $zip->addFile($tempDir . '/pass.json', 'pass.json');
            $zip->addFile($tempDir . '/manifest.json', 'manifest.json');
            $zip->addFile($tempDir . '/signature', 'signature');
            
            foreach ($this->files as $name => $path) {
                $zip->addFile($tempDir . '/' . $name, $name);
            }
            
            $zip->close();
            
            // Clean up temp files
            array_map('unlink', glob($tempDir . '/*'));
            rmdir($tempDir);
            
            $this->passFilePath = $pkpassFilename;
            return true;
        } catch (Exception $e) {
            $this->error = $e->getMessage();
            return false;
        }
    }
}

// Check if card_number parameter is provided
if (!isset($_GET['card_number'])) {
    file_put_contents($logFile, "Error: Card number parameter is missing\n", FILE_APPEND);
    die("Card number parameter is required");
}

$cardNumber = $_GET['card_number'];
file_put_contents($logFile, "Card Number: $cardNumber\n", FILE_APPEND);

// Get card details
try {
    $conn = getDbConnection();
    file_put_contents($logFile, "Database connection established\n", FILE_APPEND);
    
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.logo_url, l.reward_threshold, l.reward_description
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        file_put_contents($logFile, "Error: Invalid card number - no results found\n", FILE_APPEND);
        die("Invalid card number");
    }
    
    file_put_contents($logFile, "Card found: " . json_encode($card) . "\n", FILE_APPEND);
} catch (Exception $e) {
    file_put_contents($logFile, "Database error: " . $e->getMessage() . "\n", FILE_APPEND);
    die("Database error: " . $e->getMessage());
}

// Create a temporary directory for pass creation
$tempDirectory = __DIR__ . '/../temp';
if (!file_exists($tempDirectory)) {
    mkdir($tempDirectory, 0755, true);
    file_put_contents($logFile, "Created temp directory: $tempDirectory\n", FILE_APPEND);
} else {
    file_put_contents($logFile, "Temp directory exists: $tempDirectory\n", FILE_APPEND);
}

// Create a new CustomPKPass instance
try {
    $pass = new CustomPKPass($pemCertPath, $pemKeyPath, $pemWWDRPath);
    file_put_contents($logFile, "CustomPKPass instance created\n", FILE_APPEND);
    
    // Set pass creation temp directory
    $pass->setTempPath($tempDirectory);
    file_put_contents($logFile, "Temp path set\n", FILE_APPEND);
    
    file_put_contents($logFile, "Pass type and team identifiers will be set in data\n", FILE_APPEND);
} catch (Exception $e) {
    file_put_contents($logFile, "CustomPKPass instantiation error: " . $e->getMessage() . "\n", FILE_APPEND);
    die("CustomPKPass instantiation error: " . $e->getMessage());
}

// Create pass data
$passData = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.pointify.pk',
    'serialNumber' => $cardNumber,
    'teamIdentifier' => 'C7CH6P3GLR',
    'organizationName' => $card['location_name'],
    'description' => 'Loyalty Card for ' . $card['location_name'],
    'logoText' => $card['location_name'],
    'foregroundColor' => 'rgb(255, 255, 255)',
    'backgroundColor' => 'rgb(67, 97, 238)',
    'labelColor' => 'rgb(255, 255, 255)',
    'storeCard' => [
        'headerFields' => [
            [
                'key' => 'balance',
                'label' => 'COLLECTED',
                'value' => $card['stamp_count'] . '/' . $card['reward_threshold']
            ]
        ],
        'primaryFields' => [
            [
                'key' => 'offer',
                'label' => 'REWARD',
                'value' => 'Buy ' . ($card['reward_threshold'] - 1) . ' Get 1 FREE!'
            ]
        ],
        'secondaryFields' => [
            [
                'key' => 'name',
                'label' => 'CARDHOLDER',
                'value' => $card['name']
            ]
        ],
        'backFields' => [
            [
                'key' => 'terms',
                'label' => 'TERMS AND CONDITIONS',
                'value' => 'Present this card when making a purchase to collect stamps. Collect ' . $card['reward_threshold'] . ' stamps to get a free item.'
            ],
            [
                'key' => 'website',
                'label' => 'WEBSITE',
                'value' => 'https://pointify.com'
            ],
            [
                'key' => 'cardNumber',
                'label' => 'CARD NUMBER',
                'value' => $cardNumber
            ]
        ],
        'barcode' => [
            'format' => 'PKBarcodeFormatPDF417',
            'message' => $cardNumber,
            'messageEncoding' => 'iso-8859-1',
            'altText' => $cardNumber
        ]
    ]
];

file_put_contents($logFile, "Pass data prepared\n", FILE_APPEND);

// Set the pass data
try {
    $pass->setData($passData);
    file_put_contents($logFile, "Pass data set\n", FILE_APPEND);
} catch (Exception $e) {
    file_put_contents($logFile, "Error setting pass data: " . $e->getMessage() . "\n", FILE_APPEND);
    die("Error setting pass data: " . $e->getMessage());
}

// Add pass images
$passImagesDir = realpath(__DIR__ . '/../assets/pass_images');
file_put_contents($logFile, "Pass images directory: $passImagesDir\n", FILE_APPEND);

try {
    // Check if image files exist before adding them
    $requiredImages = [
        'icon.png', 'icon@2x.png', 'logo.png', 'logo@2x.png', 'strip.png', 'strip@2x.png'
    ];
    
    foreach ($requiredImages as $image) {
        $imagePath = $passImagesDir . '/' . $image;
        if (!file_exists($imagePath)) {
            file_put_contents($logFile, "Warning: Image file not found: $imagePath\n", FILE_APPEND);
        } else {
            $pass->addFile($imagePath, $image);
            file_put_contents($logFile, "Added image file: $image\n", FILE_APPEND);
        }
    }
} catch (Exception $e) {
    file_put_contents($logFile, "Error adding image files: " . $e->getMessage() . "\n", FILE_APPEND);
    die("Error adding image files: " . $e->getMessage());
}

// Create and output the pkpass file
try {
    file_put_contents($logFile, "Creating pass file...\n", FILE_APPEND);
    
    if($pass->create(true)) {
        file_put_contents($logFile, "Pass created successfully. Path: " . $pass->getPassFilePath() . "\n", FILE_APPEND);
        
        // Check if the pass file exists
        if (!file_exists($pass->getPassFilePath())) {
            file_put_contents($logFile, "Error: Pass file was created but does not exist at the expected path\n", FILE_APPEND);
            die("Error: Pass file was created but does not exist at the expected path");
        }
        
        // Send the pass file to the browser
        file_put_contents($logFile, "Sending pass file to browser\n", FILE_APPEND);
        
        header('Content-Description: File Transfer');
        header('Content-Type: application/vnd.apple.pkpass');
        header('Content-Disposition: attachment; filename="' . $card['location_name'] . 'Card.pkpass"');
        header('Content-Transfer-Encoding: binary');
        header('Connection: Keep-Alive');
        header('Expires: 0');
        header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
        header('Pragma: public');
        
        // Output file content
        $fileContent = file_get_contents($pass->getPassFilePath());
        if ($fileContent === false) {
            file_put_contents($logFile, "Error: Unable to read pass file content\n", FILE_APPEND);
            die("Error: Unable to read pass file content");
        }
        
        echo $fileContent;
        file_put_contents($logFile, "Pass file sent to browser\n", FILE_APPEND);
        exit;
    } else {
        $error = $pass->getError();
        file_put_contents($logFile, "Error creating pass file: $error\n", FILE_APPEND);
        die("Error creating pass file: $error");
    }
} catch (Exception $e) {
    file_put_contents($logFile, "Exception during pass creation: " . $e->getMessage() . "\n", FILE_APPEND);
    die("Exception during pass creation: " . $e->getMessage());
}
?> 