<?php
/**
 * PKPass_Dev - PHP Class for creating Apple Wallet passes for development/testing
 * 
 * A simplified class that creates technically invalid but workable PKPass files
 * for development and testing purposes, bypassing certificate validation issues.
 * NOTE: This should ONLY be used for development. Do not use in production!
 */
class PKPass_Dev {
    private $passData;
    private $passDir;
    private $passImagesDir;
    private $files = [];
    private $tempPath;
    private $lastError;
    
    /**
     * Constructor
     */
    public function __construct() {
        $this->tempPath = dirname(__DIR__) . '/temp';
        
        // Create temp directory if it doesn't exist
        if (!file_exists($this->tempPath)) {
            mkdir($this->tempPath, 0755, true);
        }
        
        // Create a unique pass directory
        $this->passDir = $this->tempPath . '/pass_' . uniqid();
        mkdir($this->passDir, 0755, true);
    }
    
    /**
     * Set pass data (json content)
     * 
     * @param array $data Pass data
     * @return PKPass_Dev
     */
    public function setData($data) {
        $this->passData = $data;
        return $this;
    }
    
    /**
     * Set path to pass images directory
     * 
     * @param string $path Path to directory with pass images
     * @return PKPass_Dev
     */
    public function setPassImagesPath($path) {
        $this->passImagesDir = $path;
        return $this;
    }
    
    /**
     * Add a file to the pass
     * 
     * @param string $path Path to the file
     * @param string $name Name to use in the pass
     * @return PKPass_Dev
     */
    public function addFile($path, $name) {
        if (!file_exists($path)) {
            $this->lastError = "File not found: $path";
            return $this;
        }
        
        $this->files[$name] = $path;
        return $this;
    }
    
    /**
     * Get the last error
     * 
     * @return string Error message
     */
    public function getError() {
        return $this->lastError;
    }
    
    /**
     * Create the PKPass file
     * 
     * @return string|false Path to the created PKPass file or false on failure
     */
    public function create() {
        // 1. Create pass.json
        if (empty($this->passData)) {
            $this->lastError = "No pass data provided";
            return false;
        }
        
        $passJson = json_encode($this->passData);
        if (!$passJson) {
            $this->lastError = "Failed to encode pass data to JSON";
            return false;
        }
        
        file_put_contents($this->passDir . '/pass.json', $passJson);
        
        // 2. Copy pass image files if set
        if (!empty($this->passImagesDir) && is_dir($this->passImagesDir)) {
            // Required images: icon.png, logo.png
            // Optional images: icon@2x.png, logo@2x.png, strip.png, strip@2x.png
            $requiredImages = ['icon.png', 'logo.png'];
            $optionalImages = ['icon@2x.png', 'logo@2x.png', 'strip.png', 'strip@2x.png', 'thumbnail.png', 'thumbnail@2x.png'];
            
            // Copy required images (will throw error if missing)
            foreach ($requiredImages as $image) {
                $imagePath = $this->passImagesDir . '/' . $image;
                if (file_exists($imagePath)) {
                    copy($imagePath, $this->passDir . '/' . $image);
                } else {
                    $this->lastError = "Required pass image missing: $image";
                    return false;
                }
            }
            
            // Copy optional images (will continue if missing)
            foreach ($optionalImages as $image) {
                $imagePath = $this->passImagesDir . '/' . $image;
                if (file_exists($imagePath)) {
                    copy($imagePath, $this->passDir . '/' . $image);
                }
            }
        }
        
        // 3. Copy any additional files
        foreach ($this->files as $fileName => $filePath) {
            copy($filePath, $this->passDir . '/' . $fileName);
        }
        
        // 4. Create the manifest.json
        $manifestEntries = [];
        
        // Get all files in the pass directory
        $files = glob($this->passDir . '/*');
        foreach ($files as $file) {
            $fileName = basename($file);
            
            // Skip the manifest and signature files
            if ($fileName !== 'manifest.json' && $fileName !== 'signature') {
                $manifestEntries[$fileName] = sha1_file($file);
            }
        }
        
        // Write the manifest.json file
        $manifestPath = $this->passDir . '/manifest.json';
        file_put_contents($manifestPath, json_encode($manifestEntries));
        
        // 5. Create a dummy signature file (hardcoded - no OpenSSL dependency)
        $signaturePath = $this->passDir . '/signature';
        
        // This is just a dummy signature content - not a real signature
        // We're using a simplified version to avoid any certificate issues
        $dummySignature = hex2bin(
            "3082028006092a864886f70d010702a082027130820" .
            "26d02010131180f32303135303631343139313934" .
            "315a3082023c308202383082017ea00302010202" .
            "100b53992c61cae4336bf7d540a3b890a300d06" .
            "092a864886f70d01010b0500301b311930170603" .
            "55040313104465762043657274696669636174653" .
            "0301e170d3235303332353130303030305a170d33" .
            "30303332353130303030305a301b3119301706035" .
            "5040313104465762043657274696669636174653" .
            "0819f300d06092a864886f70d010101050003818d" .
            "0030818902818100c52e016c01eb7d47ba05b0149" .
            "49c71f01c94da0172c0bac106e643de1f0daa2ca4" .
            "1f4a97a6b4ba4e7e14b8da61d29819c0c8ed91a84" .
            "05b9c21f8c8ddbfa3b8c46412c0b3ed81f712719b" .
            "ca24482fa7a1b77ad32b333379d2c1bcf5e0e64a3" .
            "fef5981ed2a3a190aed2edebb676257f2f9271e8a" .
            "3b1b05a8b0203010001a321301f301d0603551d0e" .
            "0416041417814c55aa31144f0b01c78deaffeaf47" .
            "3867b3300d06092a864886f70d01010b05000381" .
            "810081dc6c0708b83d9a3c534c7ffd42dbd5c3dd" .
            "c78133281c1ef8ad7adbc266e0ceebd969a1e900" .
            "ba1759ba8b00993faef04b4e6c14e7de8e8f60ad" .
            "b5ec1a40dc66a5a33fb584e541f3b8d5a19aa5c0" .
            "a0c76f51ea989aa9948a4e4ef78e98c16c60d4b6" .
            "7cc7bd9916b8cb2cbccfeeb2b3c0c0e41e66bec9" .
            "eeb65ca5afbd6de57"
        );
        
        // Write the dummy signature to the file
        file_put_contents($signaturePath, $dummySignature);
        
        // 6. Create the .pkpass file (zip)
        $serialNumber = $this->passData['serialNumber'];
        $pkpassPath = $this->tempPath . '/' . $serialNumber . '.pkpass';
        
        if (!$this->createZip($pkpassPath)) {
            return false;
        }
        
        return $pkpassPath;
    }
    
    /**
     * Send the PKPass file to the browser
     * 
     * @param string $pkpassPath Path to the PKPass file
     * @param string $filename Optional filename to use (defaults to the pass type)
     * @return void
     */
    public function send($pkpassPath, $filename = null) {
        if (!file_exists($pkpassPath)) {
            $this->lastError = "PKPass file not found: $pkpassPath";
            return false;
        }
        
        // Get file size
        $fileSize = filesize($pkpassPath);
        if ($fileSize === 0) {
            $this->lastError = "PKPass file is empty";
            return false;
        }
        
        // Clean output buffer
        if (ob_get_level()) {
            ob_end_clean();
        }
        
        // Set the appropriate filename
        if (empty($filename)) {
            // If no filename provided, use the passTypeIdentifier and serialNumber
            $filename = (!empty($this->passData['passTypeIdentifier']) ? 
                        basename($this->passData['passTypeIdentifier']) : 'pass') . 
                        (!empty($this->passData['serialNumber']) ? 
                        '_' . $this->passData['serialNumber'] : '') . '.pkpass';
        }
        
        // Sanitize the filename
        $filename = preg_replace('/[^a-zA-Z0-9_\-\.]/', '', $filename);
        if (empty($filename)) {
            $filename = 'wallet.pkpass';
        }
        
        // Set proper content-type headers
        header('Content-Description: File Transfer');
        header('Content-Type: application/vnd.apple.pkpass');
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        header('Content-Length: ' . $fileSize);
        header('Content-Transfer-Encoding: binary');
        header('Connection: Keep-Alive');
        header('Expires: 0');
        header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
        header('Pragma: public');
        
        // Output file
        readfile($pkpassPath);
        
        // Cleanup
        if (file_exists($this->passDir)) {
            // Delete the pass directory
            $files = glob($this->passDir . '/*');
            foreach ($files as $file) {
                @unlink($file);
            }
            
            // Remove the directory
            @rmdir($this->passDir);
        }
        
        exit;
    }
    
    /**
     * Create a ZIP file with all the pass contents
     * 
     * @param string $pkpassPath Path where the .pkpass file should be saved
     * @return bool Success status
     */
    private function createZip($pkpassPath) {
        // Remove any existing pkpass file
        if (file_exists($pkpassPath)) {
            unlink($pkpassPath);
        }
        
        // Try using ZipArchive first (preferred method)
        if (class_exists('ZipArchive')) {
            $zip = new ZipArchive();
            if ($zip->open($pkpassPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === TRUE) {
                // Add all files from the pass directory to the ZIP
                $files = glob($this->passDir . '/*');
                foreach ($files as $file) {
                    $fileName = basename($file);
                    $zip->addFile($file, $fileName);
                }
                
                // Close the ZIP
                if ($zip->close()) {
                    // Verify that the PKPass file exists and has content
                    if (file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
                        return true;
                    }
                }
            }
            
            // If we reached here, ZipArchive failed
            $this->lastError = "Failed to create ZIP archive using ZipArchive";
        } else {
            $this->lastError = "ZipArchive class not available";
        }
        
        // Fallback to PclZip if available
        if (file_exists(dirname(__DIR__) . '/vendor/pclzip/pclzip/pclzip.lib.php')) {
            include_once dirname(__DIR__) . '/vendor/pclzip/pclzip/pclzip.lib.php';
            
            $zip = new \PclZip($pkpassPath);
            $fileList = [];
            
            // Build the file list
            $files = glob($this->passDir . '/*');
            foreach ($files as $file) {
                $fileName = basename($file);
                $fileList[] = [
                    'filename' => $fileName,
                    'filepath' => $file,
                    'no_compression' => true // For binary files
                ];
            }
            
            // Create the archive
            if ($zip->create($fileList) !== 0) {
                return true;
            }
            
            $this->lastError = "Failed to create ZIP archive using PclZip: " . $zip->errorInfo(true);
            return false;
        }
        
        // Final fallback - try to use command line zip if available
        if (function_exists('exec')) {
            // Create a list of files to include
            $fileList = '';
            $files = glob($this->passDir . '/*');
            foreach ($files as $file) {
                $fileName = basename($file);
                $fileList .= escapeshellarg($fileName) . ' ';
            }
            
            // Change to the pass directory
            $currentDir = getcwd();
            chdir($this->passDir);
            
            // Try to create the zip file
            $command = 'zip -r ' . escapeshellarg($pkpassPath) . ' ' . $fileList . ' 2>&1';
            exec($command, $output, $returnCode);
            
            // Change back to the original directory
            chdir($currentDir);
            
            if ($returnCode === 0 && file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
                return true;
            }
            
            $this->lastError = "Failed to create ZIP archive using command line: " . implode(', ', $output);
            return false;
        }
        
        $this->lastError = "No ZIP creation method available (ZipArchive, PclZip, or exec)";
        return false;
    }
    
    /**
     * Clean up any temporary files
     */
    public function __destruct() {
        // Clean up passDir if it exists
        if (!empty($this->passDir) && file_exists($this->passDir)) {
            // Delete all files in the pass directory
            $files = glob($this->passDir . '/*');
            foreach ($files as $file) {
                @unlink($file);
            }
            
            // Remove the directory
            @rmdir($this->passDir);
        }
    }
}
?> 