<?php
/**
 * Simple PKPass Download - Direct Approach
 * 
 * This is a simplified, "just the essentials" approach to generate and 
 * download a PKPass file that should be compatible with most environments.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logFile = __DIR__ . '/../logs/simple_pkpass.log';
if (!file_exists(dirname($logFile))) {
    mkdir(dirname($logFile), 0755, true);
}
file_put_contents($logFile, "Simple PKPass Download Started: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

function log_message($message) {
    global $logFile;
    file_put_contents($logFile, date('H:i:s') . ": $message\n", FILE_APPEND);
}

// Load functions
require_once __DIR__ . '/../includes/functions.php';
log_message("Functions loaded");

// Check for card_number parameter
if (!isset($_GET['card_number'])) {
    log_message("Error: Missing card_number parameter");
    die("Card number is required");
}

$cardNumber = $_GET['card_number'];
log_message("Card number: $cardNumber");

// Get card details from database
try {
    $conn = getDbConnection();
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.reward_threshold
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        log_message("Error: Invalid card number");
        die("Invalid card number");
    }
    
    log_message("Card found: " . json_encode($card));
} catch (Exception $e) {
    log_message("Database error: " . $e->getMessage());
    die("Database error: " . $e->getMessage());
}

// Create temporary directory
$tempDir = __DIR__ . '/../temp';
if (!file_exists($tempDir)) {
    mkdir($tempDir, 0755, true);
}

// Create a unique directory for this pass
$passDir = $tempDir . '/simple_pass_' . uniqid();
mkdir($passDir, 0755, true);
log_message("Created pass directory: $passDir");

// 1. Create pass.json
$passData = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.com.example.passbooktest',
    'serialNumber' => $cardNumber,
    'teamIdentifier' => 'C7CH6P3GLR',
    'organizationName' => $card['location_name'],
    'description' => 'Loyalty Card for ' . $card['location_name'],
    'logoText' => $card['location_name'],
    'foregroundColor' => 'rgb(255, 255, 255)',
    'backgroundColor' => 'rgb(67, 97, 238)',
    'labelColor' => 'rgb(255, 255, 255)',
    'barcodes' => [
        [
            'format' => 'PKBarcodeFormatPDF417',
            'message' => $cardNumber,
            'messageEncoding' => 'iso-8859-1',
            'altText' => $cardNumber
        ]
    ],
    'barcode' => [
        'format' => 'PKBarcodeFormatPDF417',
        'message' => $cardNumber,
        'messageEncoding' => 'iso-8859-1',
        'altText' => $cardNumber
    ],
    'storeCard' => [
        'headerFields' => [
            [
                'key' => 'balance',
                'label' => 'COLLECTED',
                'value' => $card['stamp_count'] . '/' . $card['reward_threshold']
            ]
        ],
        'primaryFields' => [
            [
                'key' => 'offer',
                'label' => 'REWARD',
                'value' => 'Buy ' . ($card['reward_threshold'] - 1) . ' Get 1 FREE!'
            ]
        ],
        'secondaryFields' => [
            [
                'key' => 'name',
                'label' => 'CARDHOLDER',
                'value' => $card['name']
            ]
        ],
        'backFields' => [
            [
                'key' => 'terms',
                'label' => 'TERMS AND CONDITIONS',
                'value' => 'Present this card when making a purchase to collect stamps.'
            ],
            [
                'key' => 'cardNumber',
                'label' => 'CARD NUMBER',
                'value' => $cardNumber
            ]
        ]
    ]
];

// Write pass.json to file
file_put_contents($passDir . '/pass.json', json_encode($passData));
log_message("Created pass.json");

// 2. Copy required images
$passImagesDir = __DIR__ . '/../assets/pass_images';
$requiredImages = ['icon.png', 'logo.png'];
$optionalImages = ['icon@2x.png', 'logo@2x.png', 'strip.png', 'strip@2x.png'];

foreach ($requiredImages as $image) {
    if (file_exists($passImagesDir . '/' . $image)) {
        copy($passImagesDir . '/' . $image, $passDir . '/' . $image);
    } else {
        log_message("Error: Required image missing - $image");
        die("Required image missing: $image");
    }
}

foreach ($optionalImages as $image) {
    if (file_exists($passImagesDir . '/' . $image)) {
        copy($passImagesDir . '/' . $image, $passDir . '/' . $image);
    }
}
log_message("Copied pass images");

// 3. Create manifest.json
$manifestEntries = [];
$files = glob($passDir . '/*');
foreach ($files as $file) {
    if (basename($file) !== 'manifest.json' && basename($file) !== 'signature') {
        $manifestEntries[basename($file)] = sha1_file($file);
    }
}
file_put_contents($passDir . '/manifest.json', json_encode($manifestEntries));
log_message("Created manifest.json");

// 4. Create dummy signature
$signatureFile = $passDir . '/signature';
$dummySignature = hex2bin(
    "3082028006092a864886f70d010702a082027130820" .
    "26d02010131180f32303135303631343139313934" .
    "315a3082023c308202383082017ea00302010202" .
    "100b53992c61cae4336bf7d540a3b890a300d06" .
    "092a864886f70d01010b0500301b311930170603" .
    "55040313104465762043657274696669636174653" .
    "0301e170d3235303332353130303030305a170d33" .
    "30303332353130303030305a301b3119301706035" .
    "5040313104465762043657274696669636174653" .
    "0819f300d06092a864886f70d010101050003818d" .
    "0030818902818100c52e016c01eb7d47ba05b0149" .
    "49c71f01c94da0172c0bac106e643de1f0daa2ca4" .
    "1f4a97a6b4ba4e7e14b8da61d29819c0c8ed91a84" .
    "05b9c21f8c8ddbfa3b8c46412c0b3ed81f712719b" .
    "ca24482fa7a1b77ad32b333379d2c1bcf5e0e64a3" .
    "fef5981ed2a3a190aed2edebb676257f2f9271e8a" .
    "3b1b05a8b0203010001a321301f301d0603551d0e" .
    "0416041417814c55aa31144f0b01c78deaffeaf47" .
    "3867b3300d06092a864886f70d01010b05000381" .
    "810081dc6c0708b83d9a3c534c7ffd42dbd5c3dd" .
    "c78133281c1ef8ad7adbc266e0ceebd969a1e900" .
    "ba1759ba8b00993faef04b4e6c14e7de8e8f60ad" .
    "b5ec1a40dc66a5a33fb584e541f3b8d5a19aa5c0" .
    "a0c76f51ea989aa9948a4e4ef78e98c16c60d4b6" .
    "7cc7bd9916b8cb2cbccfeeb2b3c0c0e41e66bec9" .
    "eeb65ca5afbd6de57"
);
file_put_contents($signatureFile, $dummySignature);
log_message("Created signature file");

// 5. Create the PKPass (zip) file
$pkpassPath = $tempDir . '/' . $cardNumber . '.pkpass';
if (file_exists($pkpassPath)) {
    unlink($pkpassPath);
}

// Use ZipArchive if available
if (class_exists('ZipArchive')) {
    $zip = new ZipArchive();
    if ($zip->open($pkpassPath, ZipArchive::CREATE) === TRUE) {
        $files = glob($passDir . '/*');
        foreach ($files as $file) {
            $zip->addFile($file, basename($file));
        }
        $zip->close();
        log_message("Created PKPass file using ZipArchive: $pkpassPath");
    } else {
        log_message("Error: Could not create ZIP archive");
        die("Could not create ZIP archive");
    }
} else {
    // Fallback to command line zip if available
    log_message("ZipArchive not available, trying command line zip");
    $currentDir = getcwd();
    chdir($passDir);
    
    $cmd = "zip -r " . escapeshellarg($pkpassPath) . " .";
    exec($cmd, $output, $returnCode);
    
    chdir($currentDir);
    
    if ($returnCode !== 0) {
        log_message("Error: Could not create ZIP archive using command line");
        die("Could not create ZIP archive using command line");
    }
    
    log_message("Created PKPass file using command line zip: $pkpassPath");
}

// Verify the PKPass file exists and has content
if (!file_exists($pkpassPath) || filesize($pkpassPath) === 0) {
    log_message("Error: PKPass file is empty or does not exist");
    die("PKPass file is empty or does not exist");
}

log_message("PKPass file created: " . filesize($pkpassPath) . " bytes");

// Set headers for download
if (ob_get_level()) {
    ob_end_clean();
}

// Important headers for pkpass files
header('Content-Description: File Transfer');
header('Content-Type: application/vnd.apple.pkpass');
header('Content-Disposition: attachment; filename="' . $card['location_name'] . '_Card.pkpass"');
header('Content-Length: ' . filesize($pkpassPath));
header('Content-Transfer-Encoding: binary');
header('Connection: Keep-Alive');
header('Expires: 0');
header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
header('Pragma: public');

// Output the file
readfile($pkpassPath);

// Clean up
$files = glob($passDir . '/*');
foreach ($files as $file) {
    @unlink($file);
}
@rmdir($passDir);

log_message("Completed. File sent to browser.");
exit; 