<?php
// Test script to verify certificate configuration

// Define certificate paths
$certDir = __DIR__ . '/../certs/';
$signerCertPath = $certDir . 'signerCert.pem';
$signerKeyPath = $certDir . 'signerKey.pem';
$wwdrCertPath = $certDir . 'Apple Worldwide Developer Relations Certification Authority.pem';

// Certificate passwords
$p12Password = 'T@pple9696';  // Password for the .p12 file
$keyPassphrase = 'test';      // Passphrase used when signing the certificate

echo "Testing certificate configuration...\n\n";

// Test 1: Check if files exist
echo "1. Checking certificate files:\n";
foreach ([$signerCertPath, $signerKeyPath, $wwdrCertPath] as $path) {
    echo "   - " . basename($path) . ": " . (file_exists($path) ? "✓ Found" : "✗ Not found") . "\n";
}
echo "\n";

// Test 2: Check file permissions
echo "2. Checking file permissions:\n";
foreach ([$signerCertPath, $signerKeyPath, $wwdrCertPath] as $path) {
    if (file_exists($path)) {
        $perms = fileperms($path);
        $readable = is_readable($path);
        echo "   - " . basename($path) . ": " . 
             "Permissions: " . substr(sprintf('%o', $perms), -4) . ", " .
             "Readable: " . ($readable ? "✓ Yes" : "✗ No") . "\n";
    }
}
echo "\n";

// Test 3: Verify certificate contents
echo "3. Verifying certificate contents:\n";

// Test private key first
echo "   Testing private key decryption:\n";
if (file_exists($signerKeyPath)) {
    $pkeyData = file_get_contents($signerKeyPath);
    $privateKey = openssl_pkey_get_private($pkeyData, $keyPassphrase);
    if ($privateKey) {
        echo "     ✓ Successfully decrypted private key\n";
        openssl_pkey_free($privateKey);
    } else {
        echo "     ✗ Failed to decrypt private key: " . openssl_error_string() . "\n";
    }
}

// Test signer certificate
if (file_exists($signerCertPath)) {
    $cert = openssl_x509_parse(file_get_contents($signerCertPath));
    if ($cert) {
        echo "   - Signer Certificate:\n";
        echo "     ✓ Valid X.509 certificate\n";
        echo "     Subject: " . $cert['subject']['CN'] . "\n";
        echo "     Expires: " . date('Y-m-d H:i:s', $cert['validTo_time_t']) . "\n";
    } else {
        echo "   ✗ Invalid signer certificate format\n";
    }
}

// Test WWDR certificate
if (file_exists($wwdrCertPath)) {
    $cert = openssl_x509_parse(file_get_contents($wwdrCertPath));
    if ($cert) {
        echo "   - WWDR Certificate:\n";
        echo "     ✓ Valid X.509 certificate\n";
        echo "     Subject: " . $cert['subject']['CN'] . "\n";
        echo "     Expires: " . date('Y-m-d H:i:s', $cert['validTo_time_t']) . "\n";
    } else {
        echo "   ✗ Invalid WWDR certificate format\n";
    }
}
echo "\n";

// Test 4: Test signature creation
echo "4. Testing signature creation:\n";
try {
    // Create a test file
    $testFile = tempnam(sys_get_temp_dir(), 'pkpass_test');
    file_put_contents($testFile, 'Test content');
    
    // Try to create a signature
    $signatureFile = tempnam(sys_get_temp_dir(), 'pkpass_sig');
    
    // Get private key
    $privateKey = openssl_pkey_get_private(file_get_contents($signerKeyPath), $keyPassphrase);
    if (!$privateKey) {
        throw new Exception("Failed to load private key: " . openssl_error_string());
    }
    
    if (openssl_pkcs7_sign(
        $testFile,
        $signatureFile,
        file_get_contents($signerCertPath),
        $privateKey,
        array(),
        PKCS7_BINARY | PKCS7_DETACHED,
        $wwdrCertPath
    )) {
        echo "   ✓ Successfully created test signature\n";
    } else {
        echo "   ✗ Failed to create signature: " . openssl_error_string() . "\n";
    }
    
    // Clean up
    openssl_pkey_free($privateKey);
    unlink($testFile);
    unlink($signatureFile);
    
} catch (Exception $e) {
    echo "   ✗ Error during signature test: " . $e->getMessage() . "\n";
}

echo "\nCertificate testing complete.\n"; 