<?php
/**
 * Certificate Test for Live Server
 * 
 * This file tests Apple Wallet certificate functionality on a live server with SSL.
 * Run this file after deploying to your production environment to verify that
 * certificates can be processed correctly.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Create logs directory
$logFile = __DIR__ . '/../logs/certificate_test.log';
if (!file_exists(dirname($logFile))) {
    mkdir(dirname($logFile), 0755, true);
}

// Start logging
file_put_contents($logFile, "Certificate Test Started: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

// Function to log messages
function log_message($message) {
    global $logFile;
    file_put_contents($logFile, $message . "\n", FILE_APPEND);
    return $message;
}

// Check if we're on HTTPS
$isHttps = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') || 
           (isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == 443);

echo "<!DOCTYPE html>
<html>
<head>
    <meta charset='UTF-8'>
    <meta name='viewport' content='width=device-width, initial-scale=1.0'>
    <title>Certificate Test for Live Server</title>
    <style>
        body { font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen, Ubuntu, Cantarell, 'Open Sans', 'Helvetica Neue', sans-serif; line-height: 1.6; color: #333; max-width: 800px; margin: 0 auto; padding: 20px; }
        h1 { color: #0071e3; }
        h2 { margin-top: 30px; border-bottom: 1px solid #eee; padding-bottom: 10px; }
        .success { color: #388e3c; }
        .error { color: #c62828; }
        .warning { color: #ff8f00; }
        .info { color: #0277bd; }
        pre { background: #f5f5f5; padding: 15px; border-radius: 5px; overflow-x: auto; }
        .card { background: #fff; border-radius: 8px; box-shadow: 0 2px 10px rgba(0,0,0,0.1); padding: 20px; margin: 20px 0; }
        .button { display: inline-block; padding: 10px 15px; background: #0071e3; color: white; text-decoration: none; border-radius: 5px; margin-top: 10px; }
        .button:hover { background: #005bb5; }
    </style>
</head>
<body>
    <h1>Certificate Test for Live Server</h1>
    <p>This test will verify if your server environment can properly process Apple Wallet certificates.</p>";

// Display HTTPS status
echo "<div class='card'>";
echo "<h2>HTTPS Status</h2>";
if ($isHttps) {
    echo "<p class='success'>✓ Your site is running on HTTPS.</p>";
    log_message("HTTPS: Enabled");
} else {
    echo "<p class='error'>✗ Your site is NOT running on HTTPS. Apple Wallet requires HTTPS for production use.</p>";
    log_message("HTTPS: Disabled");
}
echo "</div>";

// Check certificate existence
echo "<div class='card'>";
echo "<h2>Certificate Files</h2>";
$certificatePath = __DIR__ . '/../certs/Certificates.p12';
$wwdrCertPath = __DIR__ . '/../certs/AppleWWDRMPCA1G1.cer';

$certExists = file_exists($certificatePath);
$certSize = $certExists ? filesize($certificatePath) : 0;
echo "<p>P12 Certificate: " . ($certExists ? 
       "<span class='success'>Found (" . $certSize . " bytes)</span>" : 
       "<span class='error'>Missing</span>") . "</p>";
log_message("P12 Certificate: " . ($certExists ? "Found ($certSize bytes)" : "Missing"));

$wwdrExists = file_exists($wwdrCertPath);
$wwdrSize = $wwdrExists ? filesize($wwdrCertPath) : 0;
echo "<p>WWDR Certificate: " . ($wwdrExists ? 
       "<span class='success'>Found (" . $wwdrSize . " bytes)</span>" : 
       "<span class='error'>Missing</span>") . "</p>";
log_message("WWDR Certificate: " . ($wwdrExists ? "Found ($wwdrSize bytes)" : "Missing"));
echo "</div>";

// Check PHP version and extensions
echo "<div class='card'>";
echo "<h2>PHP Environment</h2>";

$phpVersion = phpversion();
echo "<p>PHP Version: <span class='info'>" . $phpVersion . "</span></p>";
log_message("PHP Version: $phpVersion");

$requiredExtensions = ['openssl', 'zip', 'json'];
$allExtensionsPresent = true;

echo "<p>Required Extensions:</p>";
echo "<ul>";
foreach ($requiredExtensions as $ext) {
    $available = extension_loaded($ext);
    if (!$available) {
        $allExtensionsPresent = false;
    }
    echo "<li>" . $ext . ": " . ($available ? 
         "<span class='success'>Available</span>" : 
         "<span class='error'>Missing</span>") . "</li>";
    log_message("Extension $ext: " . ($available ? "Available" : "Missing"));
}
echo "</ul>";

// Check OpenSSL version
if (extension_loaded('openssl')) {
    $opensslVersion = OPENSSL_VERSION_TEXT;
    echo "<p>OpenSSL Version: <span class='info'>" . $opensslVersion . "</span></p>";
    log_message("OpenSSL Version: $opensslVersion");
    
    // Check supported ciphers
    $ciphers = openssl_get_cipher_methods();
    echo "<details>";
    echo "<summary>OpenSSL Supported Ciphers (" . count($ciphers) . ")</summary>";
    echo "<pre>" . implode(", ", $ciphers) . "</pre>";
    echo "</details>";
    log_message("OpenSSL Ciphers: " . count($ciphers) . " available");
}
echo "</div>";

// PKCS12 Reading Test
echo "<div class='card'>";
echo "<h2>PKCS12 Certificate Reading Test</h2>";

if ($certExists) {
    $certPassword = 'T@pple9696';
    $certData = file_get_contents($certificatePath);
    
    if ($certData !== false) {
        $result = openssl_pkcs12_read($certData, $certs, $certPassword);
        
        if ($result) {
            echo "<p class='success'>✓ Successfully read the P12 certificate!</p>";
            log_message("P12 Certificate read: Success");
            
            echo "<p>Certificate contains:</p>";
            echo "<ul>";
            foreach (array_keys($certs) as $key) {
                echo "<li>" . htmlspecialchars($key) . "</li>";
                log_message("P12 contains: $key");
            }
            echo "</ul>";
            
            // Verify certificate details
            if (isset($certs['cert'])) {
                $certInfo = openssl_x509_parse($certs['cert']);
                if ($certInfo) {
                    echo "<p>Certificate subject: " . htmlspecialchars($certInfo['subject']['CN']) . "</p>";
                    echo "<p>Valid until: " . date('Y-m-d H:i:s', $certInfo['validTo_time_t']) . "</p>";
                    
                    // Check expiration
                    $now = time();
                    if ($certInfo['validTo_time_t'] < $now) {
                        echo "<p class='error'>✗ Certificate has EXPIRED!</p>";
                        log_message("Certificate status: Expired");
                    } else {
                        $daysRemaining = floor(($certInfo['validTo_time_t'] - $now) / 86400);
                        echo "<p class='success'>✓ Certificate is valid for $daysRemaining more days.</p>";
                        log_message("Certificate status: Valid for $daysRemaining days");
                    }
                }
            }
        } else {
            $error = openssl_error_string();
            echo "<p class='error'>✗ Failed to read the P12 certificate: " . htmlspecialchars($error) . "</p>";
            log_message("P12 Certificate read: Failed - $error");
            
            echo "<p>This is a critical error that must be fixed. Possible reasons:</p>";
            echo "<ul>";
            echo "<li>The certificate password is incorrect</li>";
            echo "<li>The certificate file is corrupt</li>";
            echo "<li>OpenSSL on your server doesn't support the algorithm used in the certificate</li>";
            echo "</ul>";
            
            // Suggest exporting a new certificate
            echo "<div class='warning' style='background: #fff8e1; padding: 15px; border-radius: 5px;'>";
            echo "<p><strong>Suggestion:</strong> Try exporting a new certificate from Apple Developer Portal with explicit settings:</p>";
            echo "<ol>";
            echo "<li>Use the latest version of Keychain Access on macOS</li>";
            echo "<li>When exporting, choose 'Personal Information Exchange (.p12)' format</li>";
            echo "<li>Under options, select 'Use AES-256 Encryption' if available</li>";
            echo "<li>Make sure the password is simple (without special characters)</li>";
            echo "</ol>";
            echo "</div>";
        }
    } else {
        echo "<p class='error'>✗ Failed to read the certificate file</p>";
        log_message("P12 Certificate read: Failed - couldn't read file");
    }
} else {
    echo "<p class='error'>Certificate file not found. Please upload it first.</p>";
    log_message("P12 Certificate: Not found");
}
echo "</div>";

// WWDR Certificate Test
echo "<div class='card'>";
echo "<h2>WWDR Certificate Test</h2>";

if ($wwdrExists) {
    $wwdrData = file_get_contents($wwdrCertPath);
    
    if ($wwdrData !== false) {
        // Convert DER to PEM
        $pem = chunk_split(base64_encode($wwdrData), 64, "\n");
        $pem = "-----BEGIN CERTIFICATE-----\n" . $pem . "-----END CERTIFICATE-----\n";
        
        // Try to read the certificate
        $cert = openssl_x509_read($pem);
        
        if ($cert) {
            echo "<p class='success'>✓ Successfully read the WWDR certificate!</p>";
            log_message("WWDR Certificate read: Success");
            
            // Get certificate details
            $certInfo = openssl_x509_parse($cert);
            if ($certInfo) {
                echo "<p>Certificate details:</p>";
                echo "<ul>";
                if (isset($certInfo['subject']['CN'])) {
                    echo "<li>Subject: " . htmlspecialchars($certInfo['subject']['CN']) . "</li>";
                    log_message("WWDR Subject: " . $certInfo['subject']['CN']);
                }
                if (isset($certInfo['issuer']['CN'])) {
                    echo "<li>Issuer: " . htmlspecialchars($certInfo['issuer']['CN']) . "</li>";
                    log_message("WWDR Issuer: " . $certInfo['issuer']['CN']);
                }
                if (isset($certInfo['validTo_time_t'])) {
                    $validTo = date('Y-m-d H:i:s', $certInfo['validTo_time_t']);
                    echo "<li>Valid until: " . $validTo . "</li>";
                    log_message("WWDR Valid until: $validTo");
                    
                    // Check expiration
                    if ($certInfo['validTo_time_t'] < time()) {
                        echo "<p class='error'>✗ WWDR Certificate has EXPIRED!</p>";
                        log_message("WWDR Certificate status: Expired");
                    }
                }
                echo "</ul>";
            }
        } else {
            $error = openssl_error_string();
            echo "<p class='error'>✗ Failed to read the WWDR certificate: " . htmlspecialchars($error) . "</p>";
            log_message("WWDR Certificate read: Failed - $error");
            
            // Show download link for fresh certificate
            echo "<p>You may need to download a fresh WWDR certificate from Apple:</p>";
            echo "<p><a href='https://www.apple.com/certificateauthority/' target='_blank' class='button'>Get Apple WWDR Certificate</a></p>";
        }
    } else {
        echo "<p class='error'>✗ Failed to read the WWDR certificate file</p>";
        log_message("WWDR Certificate read: Failed - couldn't read file");
    }
} else {
    echo "<p class='error'>WWDR certificate file not found. Please upload it first.</p>";
    log_message("WWDR Certificate: Not found");
}
echo "</div>";

// Signature Creation Test
echo "<div class='card'>";
echo "<h2>Signature Creation Test</h2>";

if ($certExists && $wwdrExists && $allExtensionsPresent) {
    // Create a temporary directory for the test
    $tempDir = __DIR__ . '/../temp/test_' . uniqid();
    if (!file_exists($tempDir)) {
        mkdir($tempDir, 0755, true);
    }
    
    // Create a test file to sign
    $testFile = $tempDir . '/test.txt';
    file_put_contents($testFile, 'Test data to sign');
    log_message("Created test file for signing");
    
    // Extract cert and key from P12
    if (openssl_pkcs12_read(file_get_contents($certificatePath), $certs, $certPassword)) {
        log_message("Extracted certificate and key from P12");
        
        // Save certificate and key to temporary files
        $certFile = $tempDir . '/cert.pem';
        $keyFile = $tempDir . '/key.pem';
        file_put_contents($certFile, $certs['cert']);
        file_put_contents($keyFile, $certs['pkey']);
        
        // Convert WWDR to PEM
        $wwdrPem = chunk_split(base64_encode(file_get_contents($wwdrCertPath)), 64, "\n");
        $wwdrPem = "-----BEGIN CERTIFICATE-----\n" . $wwdrPem . "-----END CERTIFICATE-----\n";
        $wwdrFile = $tempDir . '/wwdr.pem';
        file_put_contents($wwdrFile, $wwdrPem);
        
        // Try direct PHP signing method
        $signaturePath = $tempDir . '/signature.pem';
        $result = openssl_pkcs7_sign(
            $testFile,
            $signaturePath,
            "file://$certFile",
            ["file://$keyFile", $certPassword],
            [],
            PKCS7_BINARY | PKCS7_DETACHED,
            "file://$wwdrFile"
        );
        
        if ($result && file_exists($signaturePath) && filesize($signaturePath) > 0) {
            echo "<p class='success'>✓ Successfully created a signature using PHP openssl functions!</p>";
            log_message("Signature creation: Success");
            
            // Try to extract DER data
            $signatureContent = file_get_contents($signaturePath);
            if (preg_match('~Content-Disposition:[^\n]+\s*?([A-Za-z0-9+=/\r\n]+)\s*?-----~s', $signatureContent, $matches)) {
                $derSignature = base64_decode(preg_replace('/\s+/', '', $matches[1]));
                file_put_contents($tempDir . '/signature.der', $derSignature);
                echo "<p class='success'>✓ Successfully extracted DER data from PEM signature</p>";
                log_message("DER extraction: Success");
                
                // Show signature size
                $derSize = strlen($derSignature);
                echo "<p>DER signature size: " . $derSize . " bytes</p>";
                log_message("DER signature size: $derSize bytes");
            } else {
                echo "<p class='error'>✗ Failed to extract DER data from PEM signature</p>";
                log_message("DER extraction: Failed");
            }
        } else {
            $error = openssl_error_string();
            echo "<p class='error'>✗ Failed to create signature using PHP: " . htmlspecialchars($error) . "</p>";
            log_message("Signature creation: Failed - $error");
            echo "<p>This is a critical error that must be fixed before PKPass generation will work.</p>";
            
            // Suggestions for fixing
            echo "<div class='warning' style='background: #fff8e1; padding: 15px; border-radius: 5px;'>";
            echo "<p><strong>Suggestions to fix signature issues:</strong></p>";
            echo "<ol>";
            echo "<li>Check if your web hosting supports the required OpenSSL functions</li>";
            echo "<li>Update OpenSSL to the latest version</li>";
            echo "<li>Re-export your certificates with compatible settings</li>";
            echo "<li>If possible, try using a different hosting provider with better PHP/OpenSSL support</li>";
            echo "</ol>";
            echo "</div>";
        }
        
        // Clean up temporary files
        @unlink($testFile);
        @unlink($certFile);
        @unlink($keyFile);
        @unlink($wwdrFile);
        @unlink($signaturePath);
        @unlink($tempDir . '/signature.der');
        @rmdir($tempDir);
        
        log_message("Cleaned up temporary files");
    } else {
        $error = openssl_error_string();
        echo "<p class='error'>✗ Failed to read P12 certificate for signing test: " . htmlspecialchars($error) . "</p>";
        log_message("P12 extraction for signing: Failed - $error");
    }
} else {
    echo "<p class='error'>One or more required components missing for signature test</p>";
    log_message("Signature test: Skipped - missing components");
}
echo "</div>";

// Overall Assessment
echo "<div class='card'>";
echo "<h2>Overall Assessment</h2>";

// Only consider ready for production if all tests pass
$readyForProduction = $isHttps && $certExists && $wwdrExists && $allExtensionsPresent;
$signatureTestPassed = isset($result) && $result === true;

// If signature test failed, not ready
if (isset($result) && $result !== true) {
    $readyForProduction = false;
}

if ($readyForProduction) {
    echo "<p class='success' style='font-weight: bold; font-size: 1.2em;'>✓ Your server appears ready for PKPass generation!</p>";
    log_message("Overall assessment: Ready for production");
} else {
    echo "<p class='error' style='font-weight: bold; font-size: 1.2em;'>✗ Your server is not fully configured for PKPass generation.</p>";
    echo "<p>Please address the issues highlighted above.</p>";
    log_message("Overall assessment: NOT ready for production");
}

// Recommendations
echo "<h3>Recommendations:</h3>";
echo "<ul>";

if (!$isHttps) {
    echo "<li>Enable HTTPS on your server (required for Apple Wallet)</li>";
}

if (!$certExists || !$result) {
    echo "<li>Ensure your P12 certificate is valid and the password is correct</li>";
}

if (!$wwdrExists) {
    echo "<li>Download the latest WWDR certificate from Apple</li>";
}

if (!$allExtensionsPresent) {
    echo "<li>Install required PHP extensions</li>";
}

if (isset($result) && $result !== true) {
    echo "<li>Check OpenSSL configuration and compatibility with your certificates</li>";
    echo "<li>Consider using the PKPass PHP class that includes fallback mechanisms</li>";
}

echo "</ul>";
echo "</div>";

// Additional Information
echo "<div class='card'>";
echo "<h2>Additional Information</h2>";
echo "<p>Apple requires specific formats and valid certificates for PKPass files to work properly.</p>";

echo "<h3>Required Certificate Files:</h3>";
echo "<ul>";
echo "<li><strong>Pass Type ID Certificate (.p12)</strong> - Obtained from Apple Developer Portal</li>";
echo "<li><strong>Apple Worldwide Developer Relations Certificate (.cer)</strong> - Apple's Intermediate Certificate</li>";
echo "</ul>";

echo "<h3>Helpful Resources:</h3>";
echo "<ul>";
echo "<li><a href='https://developer.apple.com/documentation/walletpasses' target='_blank'>Apple Wallet Developer Documentation</a></li>";
echo "<li><a href='https://developer.apple.com/account/resources/certificates/list' target='_blank'>Apple Developer Certificates Portal</a></li>";
echo "<li><a href='https://www.apple.com/certificateauthority/' target='_blank'>Apple Certificate Authority</a></li>";
echo "</ul>";
echo "</div>";

// Links to other test tools
echo "<div class='card'>";
echo "<h2>Additional Tests</h2>";
echo "<p><a href='test_certificate.php' class='button'>Run Basic Certificate Test</a></p>";
echo "<p><a href='simple_pkpass_test.php' class='button'>Generate Test PKPass</a></p>";
echo "<p><a href='generate_proper_pkpass.php?card_number=LC202503253423' class='button' style='background-color: #388e3c;'>Generate Real Loyalty Card PKPass</a></p>";
echo "</div>";

echo "</body>
</html>";

// Final log entry
log_message("Certificate Test Completed: " . date('Y-m-d H:i:s'));
?> 