<?php
/**
 * Ultra Simple PKPass Generator
 * 
 * This script creates a PKPass file using an in-memory approach
 * without relying on OpenSSL file operations.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../logs';
if (!file_exists($logDir)) {
    mkdir($logDir, 0755, true);
}

$logFile = $logDir . '/ultra_simple_pass.log';
file_put_contents($logFile, "Ultra Simple PKPass Started: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

function log_message($message) {
    global $logFile;
    file_put_contents($logFile, date('H:i:s') . ": $message\n", FILE_APPEND);
}

// Load functions
require_once __DIR__ . '/../includes/functions.php';
log_message("Functions loaded");

// Check for card_number parameter
if (!isset($_GET['card_number'])) {
    log_message("Error: Missing card_number parameter");
    die("Card number is required");
}

$cardNumber = $_GET['card_number'];
log_message("Card number: $cardNumber");

// Get card details from database
try {
    $conn = getDbConnection();
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.reward_threshold
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        log_message("Error: Invalid card number");
        die("Invalid card number");
    }
    
    log_message("Card found for: " . $card['name']);
} catch (Exception $e) {
    log_message("Database error: " . $e->getMessage());
    die("Database error: " . $e->getMessage());
}

// Create temporary directory
$tempDir = __DIR__ . '/../temp';
if (!file_exists($tempDir)) {
    mkdir($tempDir, 0755, true);
}

// Create a unique directory for this pass
$passDir = $tempDir . '/ultra_pass_' . uniqid();
mkdir($passDir, 0755, true);
log_message("Created pass directory: $passDir");

// Create minimal, valid pass.json
$passData = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.com.example.passbooktest', // Generic identifier that works for testing
    'serialNumber' => $cardNumber,
    'teamIdentifier' => 'ABCDEF12345', // Generic team ID
    'organizationName' => $card['location_name'],
    'description' => 'Loyalty Card',
    'foregroundColor' => 'rgb(255, 255, 255)',
    'backgroundColor' => 'rgb(60, 60, 60)',
    'labelColor' => 'rgb(255, 255, 255)',
    'logoText' => $card['location_name'],
    'barcodes' => [
        [
            'format' => 'PKBarcodeFormatPDF417',
            'message' => $cardNumber,
            'messageEncoding' => 'iso-8859-1',
            'altText' => $cardNumber
        ]
    ],
    'storeCard' => [
        'primaryFields' => [
            [
                'key' => 'member',
                'label' => 'MEMBER',
                'value' => $card['name']
            ]
        ],
        'secondaryFields' => [
            [
                'key' => 'points',
                'label' => 'STAMPS',
                'value' => $card['stamp_count'] . '/' . $card['reward_threshold']
            ]
        ]
    ]
];

// Write pass.json
file_put_contents($passDir . '/pass.json', json_encode($passData, JSON_PRETTY_PRINT));
log_message("Created pass.json");

// Create minimal PNG icons (base64 encoded 1x1 pixel transparent PNG)
$minimalPng = base64_decode('iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR42mNkYAAAAAYAAjCB0C8AAAAASUVORK5CYII=');

// Write the minimal icon and logo files
file_put_contents($passDir . '/icon.png', $minimalPng);
file_put_contents($passDir . '/logo.png', $minimalPng);
log_message("Created minimal PNG files");

// Copy actual images if they exist
$passImagesDir = __DIR__ . '/../assets/pass_images';
$imageFiles = ['icon.png', 'logo.png', 'strip.png', 'icon@2x.png', 'logo@2x.png'];

foreach ($imageFiles as $image) {
    if (file_exists($passImagesDir . '/' . $image)) {
        copy($passImagesDir . '/' . $image, $passDir . '/' . $image);
        log_message("Copied image: $image");
    }
}

// Create manifest.json
$manifestEntries = [];
$files = glob($passDir . '/*');
foreach ($files as $file) {
    if (basename($file) !== 'manifest.json' && basename($file) !== 'signature') {
        $manifestEntries[basename($file)] = sha1_file($file);
    }
}
file_put_contents($passDir . '/manifest.json', json_encode($manifestEntries, JSON_PRETTY_PRINT));
log_message("Created manifest.json");

// Create minimal signature (16 bytes of zero, which won't validate but allows the pass to be parsed)
$minimalSignature = str_repeat("\0", 16);
file_put_contents($passDir . '/signature', $minimalSignature);
log_message("Created minimal signature");

// Create the PKPass file using ZipArchive
$pkpassPath = $tempDir . '/ultra_' . $cardNumber . '.pkpass';
if (file_exists($pkpassPath)) {
    unlink($pkpassPath);
}

// Create the zip file
$success = false;

// Method 1: Try ZipArchive
if (class_exists('ZipArchive')) {
    log_message("Trying ZipArchive method");
    $zip = new ZipArchive();
    if ($zip->open($pkpassPath, ZipArchive::CREATE) === TRUE) {
        $files = glob($passDir . '/*');
        foreach ($files as $file) {
            $zip->addFile($file, basename($file));
        }
        $zip->close();
        
        if (file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
            log_message("Created PKPass using ZipArchive: " . filesize($pkpassPath) . " bytes");
            $success = true;
        } else {
            log_message("ZipArchive failed to create valid ZIP file");
        }
    } else {
        log_message("Failed to open ZIP file with ZipArchive");
    }
}

// Method 2: Try command line zip
if (!$success) {
    log_message("Trying command line zip");
    $cmd = "cd " . escapeshellarg($passDir) . " && zip -q -r " . escapeshellarg($pkpassPath) . " .";
    exec($cmd, $output, $returnVal);
    
    if ($returnVal === 0 && file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
        log_message("Created PKPass using command line zip: " . filesize($pkpassPath) . " bytes");
        $success = true;
    } else {
        log_message("Command line zip failed: " . implode("\n", $output));
    }
}

// Method 3: Manual ZIP creation using PHP
if (!$success) {
    log_message("Using PHP-based ZIP creation");
    
    // Very simple ZIP file structure
    $zipData = "";
    $files = glob($passDir . '/*');
    
    foreach ($files as $file) {
        $filename = basename($file);
        $content = file_get_contents($file);
        
        // Local file header
        $zipData .= "\x50\x4b\x03\x04"; // Local file header signature
        $zipData .= "\x0a\x00"; // Version needed to extract
        $zipData .= "\x00\x00"; // General purpose bit flag
        $zipData .= "\x00\x00"; // Compression method (none)
        $zipData .= "\x00\x00\x00\x00"; // Last mod time and date
        
        $crc = crc32($content);
        $zipData .= pack("V", $crc); // CRC-32
        $zipData .= pack("V", strlen($content)); // Compressed size
        $zipData .= pack("V", strlen($content)); // Uncompressed size
        $zipData .= pack("v", strlen($filename)); // Filename length
        $zipData .= pack("v", 0); // Extra field length
        $zipData .= $filename; // Filename
        $zipData .= $content; // File content
    }
    
    // End of central directory record
    $zipData .= "\x50\x4b\x05\x06"; // End of central directory signature
    $zipData .= "\x00\x00\x00\x00"; // Number of this disk
    $zipData .= "\x00\x00\x00\x00"; // Disk where central directory starts
    $zipData .= "\x00\x00\x00\x00"; // Number of central directory records on this disk
    $zipData .= "\x00\x00\x00\x00"; // Total number of central directory records
    $zipData .= "\x00\x00\x00\x00"; // Size of central directory
    $zipData .= "\x00\x00\x00\x00"; // Offset of start of central directory
    $zipData .= "\x00\x00"; // Comment length
    
    file_put_contents($pkpassPath, $zipData);
    
    if (file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
        log_message("Created PKPass using manual PHP zip: " . filesize($pkpassPath) . " bytes");
        $success = true;
    } else {
        log_message("Manual PHP zip creation failed");
    }
}

// Check if we were able to create the PKPass file
if (!$success) {
    log_message("Failed to create PKPass file using any method");
    die("Failed to create PKPass file. Please check the logs for details.");
}

// Clean output buffer
if (ob_get_level()) {
    ob_end_clean();
}

// Set headers for download
header('Content-Description: File Transfer');
header('Content-Type: application/vnd.apple.pkpass');
header('Content-Disposition: attachment; filename="' . basename($pkpassPath) . '"');
header('Content-Length: ' . filesize($pkpassPath));
header('Content-Transfer-Encoding: binary');
header('Connection: Keep-Alive');
header('Expires: 0');
header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
header('Pragma: public');

// Output the file
readfile($pkpassPath);

// Clean up
$files = glob($passDir . '/*');
foreach ($files as $file) {
    @unlink($file);
}
@rmdir($passDir);

log_message("Completed. File sent to browser.");
exit; 