<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Include database connection
require_once('../includes/functions.php');

// Get the request path
$requestURI = $_SERVER['REQUEST_URI'];
$requestMethod = $_SERVER['REQUEST_METHOD'];

// Parse the URL to extract the relevant parts
$path = parse_url($requestURI, PHP_URL_PATH);
$pathParts = explode('/', trim($path, '/'));

// Find the position of 'webservice' or 'customer/webservice.php' in the path
$webservicePos = array_search('webservice.php', $pathParts);
if ($webservicePos === false) {
    $webservicePos = array_search('webservice', $pathParts);
}

// Extract the parts of the path after 'webservice'
$endpointParts = array_slice($pathParts, $webservicePos + 1);

// Handle the endpoints based on Apple's Wallet Pass Web Service reference
// https://developer.apple.com/documentation/walletpasses/adding_a_web_service_to_update_passes

// Log the request for debugging
$logFile = __DIR__ . '/../logs/webservice.log';
file_put_contents($logFile, date('Y-m-d H:i:s') . " - " . $requestMethod . " " . $path . "\n", FILE_APPEND);
file_put_contents($logFile, "Body: " . file_get_contents('php://input') . "\n\n", FILE_APPEND);

// Extract endpoint components if they exist
$passTypeIdentifier = isset($endpointParts[0]) ? $endpointParts[0] : null;
$serialNumber = isset($endpointParts[1]) ? $endpointParts[1] : null;
$deviceLibraryIdentifier = isset($endpointParts[3]) ? $endpointParts[3] : null;

// Route to the appropriate handler based on the endpoint
if ($requestMethod === 'GET' && $passTypeIdentifier && $serialNumber && count($endpointParts) === 2) {
    // GET /webservice/v1/passes/{passTypeIdentifier}/{serialNumber}
    getLatestPass($passTypeIdentifier, $serialNumber);
} elseif ($requestMethod === 'GET' && $passTypeIdentifier && $deviceLibraryIdentifier && count($endpointParts) === 4 && $endpointParts[2] === 'registrations') {
    // GET /webservice/v1/devices/{deviceLibraryIdentifier}/registrations/{passTypeIdentifier}[?passesUpdatedSince=tag]
    getSerialNumbers($passTypeIdentifier, $deviceLibraryIdentifier);
} elseif ($requestMethod === 'POST' && $passTypeIdentifier && $serialNumber && $deviceLibraryIdentifier && count($endpointParts) === 4 && $endpointParts[2] === 'registrations') {
    // POST /webservice/v1/devices/{deviceLibraryIdentifier}/registrations/{passTypeIdentifier}/{serialNumber}
    registerDevice($passTypeIdentifier, $serialNumber, $deviceLibraryIdentifier);
} elseif ($requestMethod === 'DELETE' && $passTypeIdentifier && $serialNumber && $deviceLibraryIdentifier && count($endpointParts) === 4 && $endpointParts[2] === 'registrations') {
    // DELETE /webservice/v1/devices/{deviceLibraryIdentifier}/registrations/{passTypeIdentifier}/{serialNumber}
    unregisterDevice($passTypeIdentifier, $serialNumber, $deviceLibraryIdentifier);
} elseif ($requestMethod === 'POST' && count($endpointParts) === 1 && $endpointParts[0] === 'log') {
    // POST /webservice/v1/log
    logMessages();
} else {
    // Endpoint not found
    header('HTTP/1.0 404 Not Found');
    echo json_encode(['status' => 'error', 'message' => 'Endpoint not found']);
    exit;
}

/**
 * Gets the latest version of a pass
 * @param string $passTypeIdentifier Pass type identifier
 * @param string $serialNumber Serial number (card number)
 */
function getLatestPass($passTypeIdentifier, $serialNumber) {
    // Get card details
    $conn = getDbConnection();
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.logo_url, l.reward_threshold, l.reward_description
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $serialNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);

    if (!$card) {
        header('HTTP/1.0 404 Not Found');
        echo json_encode(['status' => 'error', 'message' => 'Pass not found']);
        exit;
    }

    // Check if pass exists
    $passFile = __DIR__ . '/../passes/' . $serialNumber . '.pkpass';
    
    if (!file_exists($passFile)) {
        // Generate the pass if it doesn't exist
        require_once(__DIR__ . '/generate_pass.php');
        // The generate_pass.php script will create the pass and redirect, so we shouldn't reach here
        exit;
    }

    // Check if the client provided If-Modified-Since header
    if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE'])) {
        $modifiedSince = strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']);
        $passModified = filemtime($passFile);
        
        // Check if the pass was updated since the client's version
        if ($passModified <= $modifiedSince) {
            header('HTTP/1.0 304 Not Modified');
            exit;
        }
    }

    // Set content type for the pass file
    header('Content-Type: application/vnd.apple.pkpass');
    header('Content-Disposition: attachment; filename="' . basename($passFile) . '"');
    header('Content-Length: ' . filesize($passFile));
    header('Last-Modified: ' . gmdate('D, d M Y H:i:s', filemtime($passFile)) . ' GMT');
    
    // Output the pass file
    readfile($passFile);
    exit;
}

/**
 * Gets serial numbers of passes that were updated since a timestamp
 * @param string $passTypeIdentifier Pass type identifier
 * @param string $deviceLibraryIdentifier Device library identifier
 */
function getSerialNumbers($passTypeIdentifier, $deviceLibraryIdentifier) {
    $updatedSince = isset($_GET['passesUpdatedSince']) ? $_GET['passesUpdatedSince'] : null;
    
    $conn = getDbConnection();
    $query = "
        SELECT pr.serial_number, pr.updated_at
        FROM pass_registrations pr
        JOIN loyalty_cards lc ON pr.serial_number = lc.card_number
        WHERE pr.device_library_identifier = :device_id 
        AND pr.pass_type_identifier = :pass_type
    ";
    
    $params = [
        ':device_id' => $deviceLibraryIdentifier,
        ':pass_type' => $passTypeIdentifier
    ];
    
    if ($updatedSince) {
        $query .= " AND (lc.updated_at > :updated_since OR pr.updated_at > :updated_since)";
        $params[':updated_since'] = date('Y-m-d H:i:s', strtotime($updatedSince));
    }
    
    $stmt = $conn->prepare($query);
    foreach ($params as $key => $value) {
        $stmt->bindValue($key, $value);
    }
    $stmt->execute();
    
    $serialNumbers = [];
    $lastUpdated = null;
    
    while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
        $serialNumbers[] = $row['serial_number'];
        
        // Track the most recent update time
        if (!$lastUpdated || strtotime($row['updated_at']) > strtotime($lastUpdated)) {
            $lastUpdated = $row['updated_at'];
        }
    }
    
    // Format response
    $response = ['serialNumbers' => $serialNumbers];
    if ($lastUpdated) {
        $response['lastUpdated'] = gmdate('Y-m-d\TH:i:s\Z', strtotime($lastUpdated));
    }
    
    header('Content-Type: application/json');
    echo json_encode($response);
    exit;
}

/**
 * Registers a device for push notifications
 * @param string $passTypeIdentifier Pass type identifier
 * @param string $serialNumber Serial number (card number)
 * @param string $deviceLibraryIdentifier Device library identifier
 */
function registerDevice($passTypeIdentifier, $serialNumber, $deviceLibraryIdentifier) {
    // Get request data
    $requestData = json_decode(file_get_contents('php://input'), true);
    $pushToken = isset($requestData['pushToken']) ? $requestData['pushToken'] : null;
    
    if (!$pushToken) {
        header('HTTP/1.0 400 Bad Request');
        echo json_encode(['status' => 'error', 'message' => 'Push token is required']);
        exit;
    }
    
    // Store device registration in database
    $conn = getDbConnection();
    
    // Check if registration already exists
    $stmt = $conn->prepare("SELECT id FROM pass_registrations 
                            WHERE device_library_identifier = :device_id 
                            AND pass_type_identifier = :pass_type 
                            AND serial_number = :serial_number");
    $stmt->bindParam(':device_id', $deviceLibraryIdentifier);
    $stmt->bindParam(':pass_type', $passTypeIdentifier);
    $stmt->bindParam(':serial_number', $serialNumber);
    $stmt->execute();
    
    if ($stmt->fetch()) {
        // Registration already exists
        header('HTTP/1.0 200 OK');
        echo json_encode(['status' => 'success']);
        exit;
    }
    
    // Create new registration
    $stmt = $conn->prepare("INSERT INTO pass_registrations 
                            (device_library_identifier, pass_type_identifier, serial_number, push_token, created_at, updated_at) 
                            VALUES 
                            (:device_id, :pass_type, :serial_number, :push_token, NOW(), NOW())");
    
    $stmt->bindParam(':device_id', $deviceLibraryIdentifier);
    $stmt->bindParam(':pass_type', $passTypeIdentifier);
    $stmt->bindParam(':serial_number', $serialNumber);
    $stmt->bindParam(':push_token', $pushToken);
    
    if ($stmt->execute()) {
        header('HTTP/1.0 201 Created');
        echo json_encode(['status' => 'success']);
    } else {
        header('HTTP/1.0 500 Internal Server Error');
        echo json_encode(['status' => 'error', 'message' => 'Failed to register device']);
    }
    exit;
}

/**
 * Unregisters a device from push notifications
 * @param string $passTypeIdentifier Pass type identifier
 * @param string $serialNumber Serial number (card number)
 * @param string $deviceLibraryIdentifier Device library identifier
 */
function unregisterDevice($passTypeIdentifier, $serialNumber, $deviceLibraryIdentifier) {
    // Remove device registration from database
    $conn = getDbConnection();
    $stmt = $conn->prepare("DELETE FROM pass_registrations 
                            WHERE device_library_identifier = :device_id 
                            AND pass_type_identifier = :pass_type 
                            AND serial_number = :serial_number");
    
    $stmt->bindParam(':device_id', $deviceLibraryIdentifier);
    $stmt->bindParam(':pass_type', $passTypeIdentifier);
    $stmt->bindParam(':serial_number', $serialNumber);
    
    if ($stmt->execute()) {
        header('HTTP/1.0 200 OK');
        echo json_encode(['status' => 'success']);
    } else {
        header('HTTP/1.0 500 Internal Server Error');
        echo json_encode(['status' => 'error', 'message' => 'Failed to unregister device']);
    }
    exit;
}

/**
 * Logs messages from the device
 */
function logMessages() {
    $logData = file_get_contents('php://input');
    $logFile = __DIR__ . '/../logs/devices.log';
    file_put_contents($logFile, date('Y-m-d H:i:s') . " - " . $logData . "\n", FILE_APPEND);
    
    header('HTTP/1.0 200 OK');
    echo json_encode(['status' => 'success']);
    exit;
} 