<?php
/**
 * Sends push notifications to all registered devices for a specific pass
 * This should be called whenever a loyalty card's data is updated
 * 
 * @param string $cardNumber The card number / serial number of the pass
 * @return bool True if notifications were sent successfully, false otherwise
 */
function sendPushNotifications($cardNumber) {
    // Connect to database
    $conn = getDbConnection();
    
    // Get all registered devices for this pass
    $stmt = $conn->prepare("
        SELECT device_library_identifier, push_token
        FROM pass_registrations
        WHERE serial_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    
    $devices = $stmt->fetchAll(PDO::FETCH_ASSOC);
    
    // If no devices are registered, there's nothing to do
    if (empty($devices)) {
        return true;
    }
    
    // Log the notification attempt
    $logFile = __DIR__ . '/../logs/push_notifications.log';
    file_put_contents($logFile, date('Y-m-d H:i:s') . " - Sending push notifications for card: " . $cardNumber . "\n", FILE_APPEND);
    
    // Apple Push Notification Service URL (production)
    $url = 'https://api.push.apple.com/3/device/';
    
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_2_0);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'apns-topic: pass.pointify.pk',
        'apns-push-type: background',
        'Content-Type: application/json',
        'Authorization: bearer ' . getAPNsAuthToken()
    ]);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_POST, true);
    
    // Payload for a background push to update the pass
    $payload = json_encode([
        'aps' => [
            'content-available' => 1
        ]
    ]);
    
    $success = true;
    
    // Send a push notification to each registered device
    foreach ($devices as $device) {
        curl_setopt($ch, CURLOPT_URL, $url . $device['push_token']);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        // Log the result
        file_put_contents($logFile, date('Y-m-d H:i:s') . " - Device: " . $device['device_library_identifier'] . 
                        " - Response: " . $httpCode . " - " . $response . "\n", FILE_APPEND);
        
        // If push failed, update overall success status
        if ($httpCode != 200) {
            $success = false;
        }
    }
    
    curl_close($ch);
    return $success;
}

/**
 * Get JWT Authorization token for APNs
 * 
 * @return string JWT token for APNs
 */
function getAPNsAuthToken() {
    // Your Apple Developer credentials
    $teamId = 'C7CH6P3GLR'; // Your Team ID from Apple Developer Portal
    $keyId = 'Y8H9GBCK64'; // The Key ID from your .p8 file
    $keyFile = __DIR__ . '/../certs/AuthKey_Y8H9GBCK64.p8'; // Path to your .p8 file
    
    if (!file_exists($keyFile)) {
        error_log("APNs authentication key file not found: $keyFile");
        return ""; 
    }
    
    // Read private key from file
    $privateKey = file_get_contents($keyFile);
    if (!$privateKey) {
        error_log("Failed to read APNs authentication key file");
        return "";
    }
    
    // Current time and expiry (1 hour)
    $issuedAt = time();
    $expiresAt = $issuedAt + 3600; // Token valid for 1 hour
    
    // JWT Headers
    $headers = [
        'alg' => 'ES256',
        'kid' => $keyId
    ];
    
    // JWT Payload
    $payload = [
        'iss' => $teamId,
        'iat' => $issuedAt,
        'exp' => $expiresAt
    ];
    
    // Encode headers and payload
    $base64Headers = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode(json_encode($headers)));
    $base64Payload = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode(json_encode($payload)));
    
    // Create signature
    $dataToSign = $base64Headers . '.' . $base64Payload;
    
    // Get the private key resource
    $pkeyid = openssl_pkey_get_private($privateKey);
    if (!$pkeyid) {
        error_log("Failed to get private key: " . openssl_error_string());
        return "";
    }
    
    // Create the signature
    $success = openssl_sign($dataToSign, $signature, $pkeyid, 'sha256');
    openssl_pkey_free($pkeyid);
    
    if (!$success) {
        error_log("Failed to create signature: " . openssl_error_string());
        return "";
    }
    
    // Encode the signature
    $base64Signature = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode($signature));
    
    // Create JWT
    $jwt = $base64Headers . '.' . $base64Payload . '.' . $base64Signature;
    
    return $jwt;
} 